
CREATE OR REPLACE PACKAGE AUXILIAR_PQ AS
  -- Devuleve 1 si el NIF es correcto, 0 sin.
  FUNCTION NIF_valido (p_nif in varchar2) RETURN number;
  -- Devuelve 1 si el dispositivo es un porttil, 0 sino
  FUNCTION es_un_portatil(p_cod_disp IN number) RETURN number;
  -- Devuelve 1 si el dispositivo a sufrido alguna avera, 0 sino
  FUNCTION ha_tenido_averia (p_cod_disp IN number) RETURN number;
  -- Devuelve el ID del departamento asignado al usuario
  FUNCTION departamento_asignado(p_usuario IN number) return number;
  -- Devuleve el valor econnico del dispositivo (precio de compra)
  FUNCTION valor_econ_dispositivo(p_cod_disp IN number) RETURN number;
  -- Devuelve el propietario del dispositivo. El cdigo de dispositivo debe existir
  FUNCTION empleado_prop_dispositivo (p_cod_disp IN number) RETURN number;
  -- Devuelve la cantidad de dispositivos asignados al empleado indicado
  FUNCTION disp_asignados (p_empleado IN number) RETURN number;
    -- Devuelve la cantidad de dispositivos del tipo indicado asignados al empleado seleccionado
  FUNCTION disp_asignados_tipo (p_empleado IN number,p_tipo_prod IN number) RETURN number;
  -- Devuelve el tipo de dispositivo
  FUNCTION tipo_producto (p_cod_disp IN number) return number;
  -- Devuelve el nmero de dispositivos de un tipo que tiene el departamento indicado
  FUNCTION Num_disp_en_departamento(p_cod_dep IN number,p_cod_tipo_disp IN number) RETURN number;
END AUXILIAR_PQ;
/


CREATE OR REPLACE PACKAGE BODY AUXILIAR_PQ AS

 -- https://es.wikibooks.org/wiki/Algoritmo_para_obtener_la_letra_del_NIF
  FUNCTION NIF_valido (p_nif in varchar2) RETURN number IS
    v_nif_sin_letra varchar2(20);
    v_nif_numero number;
    v_nif_con_letra varchar2(20);
  BEGIN
    IF (LENGTH(p_nif) <> 9) THEN
     return 0;
    ELSE
      v_nif_sin_letra := SUBSTR(p_nif, 1, 8);
      v_nif_numero := TO_NUMBER(v_nif_sin_letra);
      v_nif_con_letra := v_nif_sin_letra||SUBSTR('TRWAGMYFPDXBNJZSQVHLCKET',MOD(v_nif_numero,23)+1,1);
      IF (TRIM(v_nif_con_letra) = TRIM(p_nif)) THEN RETURN 1; ELSE RETURN 0; END IF;
    END IF;
  END NIF_valido;

  FUNCTION es_un_portatil(p_cod_disp IN number) RETURN number IS
    num_disp_port number;
  BEGIN
    BEGIN
      SELECT COUNT(cod_disp)
      INTO num_disp_port
      FROM INV_DISPOSITIVO d, INV_PRODUCTO p
      WHERE d.cod_producto = p.cod_prod and p.cod_tipo_prod = '1' and d.cod_disp=p_cod_disp; --tipo producto es 1=portatil
    END;
    if (num_disp_port > 0) then return 1; else return 0; end if;
  END;
  
  FUNCTION ha_tenido_averia (p_cod_disp IN number) RETURN number is
    v_total_averias number;
  BEGIN
    BEGIN
      SELECT COUNT(cod_disp)
      INTO v_total_averias
      FROM INV_MOVIMIENTO
      WHERE cod_tipo_mov = '5' and cod_disp = p_cod_disp;
    END;
    IF (v_total_averias >=1) THEN  RETURN 1; ELSE RETURN 0; END IF;
  END ha_tenido_averia; 
  
  
  FUNCTION departamento_asignado(p_usuario IN number) return number IS
    v_cod_departamento number;
  BEGIN
      SELECT cod_dep_emp
      INTO v_cod_departamento
      FROM INV_EMPLEADO
      WHERE cod_emp = p_usuario;
      RETURN v_cod_departamento;
  END departamento_asignado;
  
  -- PRE: El dispositivo indicado ha de existir
  FUNCTION valor_econ_dispositivo(p_cod_disp IN number) RETURN number IS
    v_importe number;
  BEGIN
      SELECT precio_compra_disp
      INTO v_importe
      FROM INV_DISPOSITIVO
      WHERE cod_disp = p_cod_disp;
      return v_importe;
  END valor_econ_dispositivo;
  
  
  FUNCTION empleado_prop_dispositivo (p_cod_disp IN number) RETURN number IS
    v_empleado_propietario number;
  BEGIN
    SELECT cod_empleado
    INTO v_empleado_propietario
    FROM INV_DISPOSITIVO
    WHERE cod_disp = p_cod_disp;
    RETURN v_empleado_propietario;
  END empleado_prop_dispositivo;
  
  FUNCTION disp_asignados (p_empleado IN number) RETURN number IS
    v_total_disp number;
  BEGIN
    -- Nota: los dispositivos averiados siguen estando asignados al propietario
    -- Si el dispositivo se da de baja se debera retirar el usuario al que est asignado
    BEGIN
      SELECT COUNT(cod_empleado) INTO v_total_disp FROM INV_DISPOSITIVO WHERE cod_empleado = p_empleado;
    END;
    return v_total_disp;
  END disp_asignados;
  
  -- Devuelve el cdigo del tipo del producto al que se refiere el dispositivo
  FUNCTION tipo_producto (p_cod_disp IN number) return number IS
    v_tipo number;
  BEGIN
    BEGIN
      SELECT p.cod_tipo_prod
      INTO v_tipo
      FROM INV_DISPOSITIVO d, INV_PRODUCTO p
      WHERE d.cod_producto = p.cod_prod and d.cod_disp=p_cod_disp; 
    END;
    return v_tipo;
  END tipo_producto;
  
  FUNCTION disp_asignados_tipo (p_empleado IN number,p_tipo_prod IN number) RETURN number IS
    v_total_disp number;
  BEGIN
    -- Nota: los dispositivos averiados siguen estando asignados al propietario
    BEGIN
      SELECT COUNT(cod_disp) 
      INTO v_total_disp 
      FROM INV_DISPOSITIVO d, INV_PRODUCTO p 
      WHERE d.cod_producto = p.cod_prod and d.cod_empleado = p_empleado and p.cod_tipo_prod = p_tipo_prod;
    END;
    return v_total_disp;
  END;
    
  FUNCTION Num_disp_en_departamento(p_cod_dep IN number,p_cod_tipo_disp IN number) RETURN number IS
    v_num_disp number := 0;
	-- Recorremos con un cursor los empleados del departamento contabilizando los dispositivos asignados
  BEGIN
    FOR empleado_rec IN (SELECT cod_emp FROM INV_EMPLEADO WHERE cod_dep_emp = p_cod_dep)
    LOOP
        v_num_disp := v_num_disp + disp_asignados_tipo(empleado_rec.cod_emp,p_cod_tipo_disp);
    END LOOP;
    RETURN v_num_disp;
  END;
END AUXILIAR_PQ;
/

