/*PACKAGE BODY ESTADISTICAS_DWH*/
CREATE OR REPLACE
PACKAGE BODY ESTADISTICAS_DWH_PQ
AS
  -- Consulta: Q1: Dispositivos comprados por ao
  -- Actualizacin: PROCEDURE ALTA_MOVIMIENTO
  -- Condiciones: where tipo_mov = 'ALTA'
  -- Post-condicin:Incrementa en uno el nmero de dispositivos comprados, en el ao indicado. Si no hay datos para ese ao se aade.
  -- Nota: Slo consideramos las altas, entendemos que para esta estadstica no interesa controlar las bajas
	PROCEDURE EST_Nuevo_dispositivo(
		p_anyo IN INTEGER)
	IS
	  n_any NUMBER;
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN
	  -- Verificamos si existe estadsticas de ao indicado
	  BEGIN
		SELECT COUNT(anyo) INTO n_any FROM R1 WHERE anyo = p_anyo;
	  END;
	  -- Si no existe el ao, se inserta
	  IF (n_any = 0) THEN
		INSERT INTO R1 (anyo,importe_orden_inversion,num_disp_comprados,num_total_averias)
		  VALUES (p_anyo,0,0,0);
	  END IF;
	  
	  UPDATE R1 SET Num_disp_comprados = Num_disp_comprados + 1 WHERE anyo = p_anyo;
	 
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nuevo dispositivo]','','OK',USER);
		
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nuevo dispositivo]','',error_desc,USER);
	  COMMIT;
	  
	END EST_Nuevo_dispositivo;
	
	-- Consulta: Q2: Porcentaje porttiles no averiados
	-- Actualizacin: PROCEDURE ALTA_MOVIMIENTO
	-- Condiciones: where tipo_mov = 'ASIGNACION' and TIPO_DISP = 'Porttil'
	-- Post-condicin: Incrementa en uno el nmero total de porttiles.

	PROCEDURE EST_Nuevo_portatil IS 
	  num_filas number := 0;
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN

	  BEGIN
		select count(*) into num_filas from R6;
	  END;
	  if (num_filas = 0) then --inicializa tablas estadsticas  
		INSERT INTO R6 VALUES (0,0);
	  end if;
	  UPDATE R6 SET num_portatiles = num_portatiles + 1;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nuevo porttil introducido]',NULL,'OK',USER);
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Error al introducir nuevo porttil]',NULL,error_desc,USER);
	  COMMIT;
	END;
	
	-- Consulta: Q2: Porcentaje porttiles no averiados
	-- Actualizacin: PROCEDURE ALTA_MOVIMIENTO
	-- Condiciones: where tipo_mov = 'BAJA' and TIPO_DISP = 'Porttil'
	-- Post-condicin: Decrementa en uno el nmero total de porttiles.
	-- Nota:
	PROCEDURE EST_Baja_portatil
	IS
	BEGIN
	  UPDATE R6 SET num_portatiles = num_portatiles - 1;
	END;
	-- Consulta: Q2: Porcentaje porttiles no averiados
	-- Actualizacin: PROCEDURE INV_GEST_DISP_ALTA_PR
	-- Condiciones: where tipo_mov = 'AVERIA' and TIPO_DISP = 'Porttil'. Comprueba si ese portatil ha sufrido avera antes
	-- Post-condicin: Incrementa contador de portatiles con avera
	-- Nota:
	PROCEDURE EST_Nueva_Averia_portatil(p_cod_disp IN NUMBER)
	IS
	  total_averias NUMBER; --0 es que no, <=1 es que s
	BEGIN
	  BEGIN
		SELECT COUNT(cod_disp)
		INTO total_averias
		FROM INV_MOVIMIENTO
		WHERE cod_disp   = p_cod_disp
		AND cod_tipo_mov = '5'; --en reparacin
	  END;
	  IF (total_averias = 1) THEN --solo la averia recien introducida
		  --Esto portatil nunca ha estado averiado y si se suma 1 nuevo averiado
		  UPDATE R6 SET num_portatiles_averiados = num_portatiles_averiados + 1;
	  END IF;
	  
		INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva averia portatil]',p_cod_disp,'OK',USER);
		
	  COMMIT;
	END;
	
	-- Consulta: Q3: Importe total de los pedidos reallizados en un ao ligadas a una orden de inversin
	-- Actualizacin: PROCEDURE INV_GEST_PED_APRUEBA_COMPRA_PR
	-- Condiciones: where p_tipo_gasto = 'I' (orden de inversin)
	-- Post-condicin: Incrementa el acumulado de ese ao de inversiones realizadas
	-- Nota:
	PROCEDURE EST_Actualiza_importe_inv(
		p_anyo    IN INTEGER,
		p_importe IN NUMBER)
	IS
	  parametrosIN VARCHAR2(1000);
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  n_any        NUMBER;
	  msg          VARCHAR2(200);
	BEGIN
	  -- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_anyo: '||p_anyo||', p_importe: '||p_importe;
	  BEGIN
		SELECT COUNT(*) INTO n_any FROM R1 WHERE anyo = p_anyo;
	  END;
	  -- Si no existen el ao, se inserta
	  IF (n_any = 0) THEN
		INSERT INTO R1 (anyo,importe_orden_inversion,num_disp_comprados,num_total_averias)
		  VALUES (p_anyo,0,0,0);
	  END IF;
	  
	  UPDATE R1
	  SET importe_orden_inversion = importe_orden_inversion + p_importe
	  WHERE anyo                  = p_anyo;
	  -- Actualizamos el log con la accin realizada
	  INSERT
	  INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log )
		VALUES(log_seq.nextval,SYSDATE,'([EST: Actualiza Importe]',parametrosIN,msg|| 'OK',USER);
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Actualiza Importe]',parametrosIN,error_desc,USER);
	  COMMIT;
	END EST_Actualiza_importe_inv;


	-- Consulta: Q13: Nmero de averias registradas en un ao concreto
	-- Actualizacin: PROCEDURE 
	-- Condiciones: ???where p_tipo_gasto = 'I' (orden de inversin)
	-- Post-condicin: Incrementa el acumulado de ese ao de averas
	-- Nota:
	PROCEDURE EST_Nueva_averia (p_anyo IN integer) IS
	  n_any number;
	  parametrosIN VARCHAR2(1000);
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN
	  -- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_anyo: '||p_anyo;
	   -- Verificamos si existe estadsticas de ao indicado
	  BEGIN
		SELECT COUNT(anyo) INTO n_any FROM R1 WHERE anyo = p_anyo;
	  END;
	  -- Si no existen el ao, se inserta
	  IF (n_any = 0) THEN
		INSERT INTO R1 (anyo,importe_orden_inversion,num_disp_comprados,num_total_averias)
		  VALUES (p_anyo,0,0,0);
	  END IF;
	  
	  UPDATE R1
	  SET Num_total_averias = Num_total_averias + 1
	  WHERE anyo    = p_anyo;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva avera]',parametrosIN,'OK',USER);
	  COMMIT;
	  
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva avera]',parametrosIN,error_desc,USER);
	  COMMIT;
	END EST_Nueva_averia;



	PROCEDURE EST_Nuevo_dispositivo_en_uso (p_dep IN integer)  is
	  n_dep number;
	  parametrosIN VARCHAR2(1000);
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN
	 -- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_dep: '||p_dep;
	  BEGIN
		SELECT COUNT(cod_dep) INTO n_dep FROM R2 WHERE cod_dep = p_dep;
	  END;
	  -- Si no existen el departamento, se inserta
	  IF (n_dep = 0) THEN
		INSERT INTO R2 (cod_dep,Num_disp_activos,Valor_econ_inventario_activo,Num_disp_diferente,Total_reasignaciones)
		  VALUES (p_dep,0,0,0,0);
	  END IF;
	  
	  UPDATE R2
	  SET Num_disp_activos = Num_disp_activos + 1
	  WHERE cod_dep = p_dep;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
	  VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva dispositivo en uso]',parametrosIN,'OK',USER);
	  COMMIT;
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva dispositivo en uso]',parametrosIN,error_desc,USER);
	  COMMIT;
	   
	END EST_Nuevo_dispositivo_en_uso;




	PROCEDURE EST_Retira_dispositivo_en_uso (p_dep IN integer) IS
	  n_dep number;
	  parametrosIN VARCHAR2(1000);
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN
	 -- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_dep: '||p_dep;
	  BEGIN
		SELECT COUNT(cod_dep) INTO n_dep FROM R2 WHERE cod_dep = p_dep;
	  END;
	  -- Si no existen el departamento, se inserta
	  IF (n_dep = 0) THEN error_desc := 'No existe el departamento'; raise e_error; END IF;
	  
	  UPDATE R2
	  SET Num_disp_activos = Num_disp_activos - 1
	  WHERE cod_dep = p_dep;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
	  VALUES(log_seq.nextval,SYSDATE,'([EST: Retira dispositivo en uso]',parametrosIN,'OK',USER);
	  COMMIT;
	  
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Retira dispositivo en uso]',parametrosIN,error_desc,USER);
	  COMMIT;
	END EST_Retira_dispositivo_en_uso;


	PROCEDURE EST_Nueva_reasignacion (p_dep IN integer) IS
	   n_dep number;
	  parametrosIN VARCHAR2(1000);
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN
	 -- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_dep: '||p_dep;
	  BEGIN
		SELECT COUNT(cod_dep) INTO n_dep FROM R2 WHERE cod_dep = p_dep;
	  END;

	  IF (n_dep = 0) THEN error_desc := 'No existe el departamento'; raise e_error; END IF;
	  
	  UPDATE R2
	  SET total_reasignaciones = total_reasignaciones + 1
	  WHERE cod_dep = p_dep;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
	  VALUES(log_seq.nextval,SYSDATE,'([EST: Reasignacin]',parametrosIN,'OK',USER);
	  COMMIT;
	  
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Reasignacin]',parametrosIN,error_desc,USER);
	  COMMIT;
	END EST_Nueva_reasignacion;


	-- Procedimiento estadstico Q6
	PROCEDURE EST_Suma_valor_inv_activo (p_dep IN number, p_importe in number) IS
	  n_dep number;
	  parametrosIN VARCHAR2(200);
	  error_desc   VARCHAR2(200); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN
	 -- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_dep: '||p_dep||' p_importe:'||p_importe;
	  BEGIN
		SELECT COUNT(cod_dep) INTO n_dep FROM R2 WHERE cod_dep = p_dep;
	  END;
	  -- Si no existen el departamento, se inserta
	  IF (n_dep = 0) THEN
		INSERT INTO R2 (cod_dep,Num_disp_activos,Valor_econ_inventario_activo,Num_disp_diferente,Total_reasignaciones)
		  VALUES (p_dep,0,0,0,0);
	  END IF;
	  
	  UPDATE R2
	  SET valor_econ_inventario_activo = valor_econ_inventario_activo + p_importe
	  WHERE cod_dep = p_dep;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
	  VALUES(log_seq.nextval,SYSDATE,'([EST: Suma valor dispositivo]',parametrosIN,'OK',USER);
	  COMMIT;
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Suma valor dispositivo]',parametrosIN,error_desc,USER);
	  COMMIT;
	END EST_Suma_valor_inv_activo;


	PROCEDURE EST_Resta_valor_inv_activo (p_dep IN number, p_importe in number) IS
	  n_dep number;
	  parametrosIN VARCHAR2(200);
	  error_desc   VARCHAR2(200); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN
	-- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_dep: '||p_dep;
	  BEGIN
		SELECT COUNT(cod_dep) INTO n_dep FROM R2 WHERE cod_dep = p_dep;
	  END;
	  -- Si no existen el departamento, se inserta
	  IF (n_dep = 0) THEN
		INSERT INTO R2 (cod_dep,Num_disp_activos,Valor_econ_inventario_activo,Num_disp_diferente,Total_reasignaciones)
		  VALUES (p_dep,0,0,0,0);
	  END IF;
	  
	  UPDATE R2
	  SET valor_econ_inventario_activo = valor_econ_inventario_activo - p_importe
	  WHERE cod_dep = p_dep;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
	  VALUES(log_seq.nextval,SYSDATE,'([EST: Suma valor dispositivo]',parametrosIN,'OK',USER);
	  COMMIT;
	  
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Suma valor dispositivo]',parametrosIN,error_desc,USER);
	  COMMIT;
	END EST_Resta_valor_inv_activo;



	  -- Procedimiento para dar soporte a Q4
	  --Incrementa el nmero de pedidos contabilizados como servidos por ese proveedor y los dias que tard en entregarlo
	PROCEDURE EST_Nuevo_pedido_servido(p_provee IN number, p_dias IN integer) IS
		  n_provee number;
		  parametrosIN VARCHAR2(200);
		  error_desc   VARCHAR2(200); -- descripcion del error
		  e_error      EXCEPTION;      -- error controlado
	BEGIN
	  -- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_provee: '||p_provee||' p_dias: '||p_dias;
	  BEGIN
		SELECT COUNT(p_provee) INTO n_provee FROM R3 WHERE cod_prov = p_provee;
	  END;
	  -- Si no existen el proveedor, se inserta
	  IF (n_provee = 0) THEN
		INSERT INTO R3 (cod_prov, total_pedidos_servidos,dias_delay) VALUES (p_provee,0,0);
	  END IF;
	  
	  UPDATE R3
	  SET  total_pedidos_servidos =  total_pedidos_servidos + 1, dias_delay = dias_delay + p_dias
	  WHERE cod_prov = p_provee;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
	  VALUES(log_seq.nextval,SYSDATE,'([EST: Pedido servido]',parametrosIN,'OK',USER);
	  
	  COMMIT;
	  
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN  error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ;  END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Error pedido servido]',parametrosIN,error_desc,USER);
	  COMMIT;
	END EST_Nuevo_pedido_servido;

	-- Procedimientos estadstico Q9
	PROCEDURE EST_Nuevo_empleado IS
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  v_filas number; -- contador para saber si la estadstica est inicializada
	BEGIN
	  -- Verificamos si existe estadsticas de ao indicado
	  BEGIN
		SELECT COUNT(total_empleados) INTO v_filas FROM R4;
	  END;
	  -- Si no existen datos, se inserta con los datos inicializados
	  IF (v_filas = 0) THEN
		INSERT INTO R4 VALUES (0,0,0);
	  END IF;
	  UPDATE R4 SET total_empleados = total_empleados + 1;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nuevo empleado]','Sin parmetros','OK',USER);
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Error nuevo empleado]','Sin parmetros',error_desc,USER);
	  COMMIT;
	END EST_Nuevo_empleado;



	PROCEDURE EST_Baja_empleado IS
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	BEGIN

	  UPDATE R4 SET total_empleados = total_empleados - 1;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Baja empleado]',NULL,'OK',USER);
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Error baja empleado]',NULL,error_desc,USER);
	  COMMIT;
	END EST_Baja_empleado;

	-- Ejecutar tras asignar un dispositivo a un empleado
	PROCEDURE EST_Nueva_asignacion_disp (p_cod_emp IN INV_EMPLEADO.cod_emp%TYPE) IS
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  v_num_disp  number;
	BEGIN
	  BEGIN
		v_num_disp := AUXILIAR_PQ.disp_asignados(p_cod_emp);
	  END;

	  if (v_num_disp = 1) then 
		-- solo dispone del dispositivo que se le acaba de asignar. Actualiza, por tanto, el nuevo empleado con dispositivo asignado
		UPDATE R4 SET empleados_con_dispositivo = empleados_con_dispositivo + 1;
	  end if;
	  if (v_num_disp = 6) then
		-- al asignarle el dispositivo pasa de los 5 dispositivos, actualizamos estadstica
		UPDATE R4 SET empleados_mas_de_5_disp = empleados_mas_de_5_disp + 1;
	  end if;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva asignacin de dispositivo]','Sin parmetros','OK',USER);
	  COMMIT;
	  
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Nueva asignacin de dispositivo]','Sin parmetros',error_desc,USER);
	  COMMIT;
	END EST_Nueva_asignacion_disp;


	PROCEDURE EST_Retirada_dipositivo (p_cod_emp IN INV_EMPLEADO.cod_emp%TYPE) IS
	 error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  v_num_disp  number;
	BEGIN
	  BEGIN
		v_num_disp := AUXILIAR_PQ.disp_asignados(p_cod_emp);
	  END;

	  if (v_num_disp = 0) then 
		-- Al retirarle el dispositivo se queda sin
		UPDATE R4 SET empleados_con_dispositivo = empleados_con_dispositivo - 1;
	  end if;
	  if (v_num_disp = 5) then
		-- al asignarle el dispositivo baja de los 6 asignados, actualizamos estadstica
		UPDATE R4 SET empleados_mas_de_5_disp = empleados_mas_de_5_disp - 1;
	  end if;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Dispositivo retirado]','Sin parmetros','OK',USER);
	  COMMIT;
	  
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Dispositivo retirado]','Sin parmetros',error_desc,USER);
	  COMMIT;
	END EST_Retirada_dipositivo;

	-- Procedimientos estadsticos Q10
	PROCEDURE EST_disp_almacenado(p_cod_disp IN INV_DISPOSITIVO.cod_disp%TYPE) IS
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  v_filas number; -- contador para saber si la estadstica est inicializada
	  v_tipo_prod number;
	  parametrosIN VARCHAR2(200);
	BEGIN
		-- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_cod_disp: '||p_cod_disp;
	  
	  -- Averiaguamos el tipo de producto almacenado
	  v_tipo_prod := AUXILIAR_PQ.tipo_producto (p_cod_disp);
	  --parametrosIN := parametrosIN || '-' || v_tipo_prod; -- PROVISIONAL!!!!
	  -- Verificamos si existe estadsticas del tipo de producto
	  BEGIN
		SELECT COUNT(*) INTO v_filas FROM R5 WHERE tipo_dispositivo=v_tipo_prod;
	  END;
	  -- Si no existen datos, se inserta con los datos inicializados para ese tipo de producto
	  IF (v_filas = 0) THEN
		INSERT INTO R5 VALUES (v_tipo_prod,0);
	  END IF;
	  
	  -- actualizamos 
	  UPDATE R5 SET unidades_pendientes_asignar = unidades_pendientes_asignar + 1 WHERE tipo_dispositivo=v_tipo_prod;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Dispositivo almacenado]',parametrosIN,'OK',USER);
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Dispositivo almacenado]',parametrosIN,error_desc,USER);
	  COMMIT; 
	END EST_disp_almacenado;


	PROCEDURE EST_disp_entregado(p_cod_disp IN INV_DISPOSITIVO.cod_disp%TYPE) IS
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  v_filas number; -- contador para saber si la estadstica est inicializada
	  v_tipo_prod number;
	  parametrosIN VARCHAR2(200);
	BEGIN
		-- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_cod_disp: '||p_cod_disp;
	  
	  -- Averiaguamos el tipo de producto almacenado
	  v_tipo_prod := AUXILIAR_PQ.tipo_producto (p_cod_disp);
	  -- actualizamos 
	  UPDATE R5 SET unidades_pendientes_asignar = unidades_pendientes_asignar - 1 WHERE tipo_dispositivo=v_tipo_prod;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Dispositivo entregado]',parametrosIN,'OK',USER);
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Dispositivo entregado]',parametrosIN,error_desc,USER);
	  COMMIT; 
	END;


	-- Procedimientos estadstico Q11
	PROCEDURE EST_suma_disp_diferente(p_dep IN INV_DEPARTAMENTO.cod_dep%TYPE, p_cod_disp IN number) IS
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  parametrosIN VARCHAR2(200);
	  n_dep number; -- contador para saber si la estadstica est inicializada
	  v_num number;
	BEGIN
		-- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_dep: '||p_dep||', p_cod_disp: '||p_cod_disp;

	  -- Calculamos cuantos dispositivos del tipo del producto en cuestin, tiene el departamento seleccionado
	  v_num := AUXILIAR_PQ.Num_disp_en_departamento(p_dep, AUXILIAR_PQ.tipo_producto(p_cod_disp));
	  parametrosIN := parametrosIN || '-' ||v_num;
	  -- Si es el primer dispositivo de ese tipo para ese departamento, se incrementa el contador de dispositivos diferentes
	  IF (v_num = 1) THEN 
		BEGIN
		  SELECT COUNT(cod_dep) INTO n_dep FROM R2 WHERE cod_dep = p_dep;
		END;
		-- Si no existen el departamento, se inserta
		IF (n_dep = 0) THEN
		  INSERT INTO R2 (cod_dep,Num_disp_activos,Valor_econ_inventario_activo,Num_disp_diferente,Total_reasignaciones)
			VALUES (p_dep,0,0,0,0);
		END IF;
	  
		-- actualizamos el contador de dispositivos diferentes
		UPDATE R2 SET num_disp_diferente = num_disp_diferente + 1 WHERE cod_dep=p_dep;
	  END IF;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Aade dispositivo tipo diferente]',parametrosIN,'OK',USER);
		
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Aade dispositivo tipo diferente]',parametrosIN,error_desc,USER);
	  COMMIT; 
	END EST_suma_disp_diferente;



	PROCEDURE EST_resta_disp_diferente(p_dep IN INV_DEPARTAMENTO.cod_dep%TYPE,p_cod_disp IN number) IS
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  parametrosIN VARCHAR2(200);
	  v_num number;
	BEGIN
		-- Concatenamos los parametros de entrada para el LOG
	  parametrosIN := 'p_dep: '||p_dep||', p_cod_disp: '||p_cod_disp;
	   -- Calculamos cuantos dispositivos del tipo del producto en cuestin, tiene el departamento seleccionado
	  v_num := AUXILIAR_PQ.Num_disp_en_departamento(p_dep, AUXILIAR_PQ.tipo_producto(p_cod_disp));
	  parametrosIN := parametrosIN || '-' ||v_num;
	  -- Si es el primer dispositivo de ese tipo para ese departamento, se incrementa el contador de dispositivos diferentes
	  IF (v_num = 0) THEN --revisar!!!
		-- actualizamos el contador de dispositivos diferentes
		UPDATE R2 SET num_disp_diferente = num_disp_diferente - 1 WHERE cod_dep=p_dep;
	  END IF;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Resta dispositivo tipo diferente]',parametrosIN,'OK',USER);
		
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Resta dispositivo tipo diferente]',parametrosIN,error_desc,USER);
	  COMMIT; 
	END EST_resta_disp_diferente;


	PROCEDURE EST_Calcula_averias IS
	  error_desc   VARCHAR2(1000); -- descripcion del error
	  e_error      EXCEPTION;      -- error controlado
	  parametrosIN VARCHAR2(200);
	  v_num number;
	BEGIN

	  -- Vacamos la tabla R7 para que no contenga ningn dato
	  delete from R7; 
	  -- En R7 se tendrn los 10 usuarios con ms averas
	  INSERT INTO R7 
	  SELECT USU_ORIGEN_MOV,COUNT(USU_ORIGEN_MOV) mis_averias FROM INV_MOVIMIENTO WHERE COD_TIPO_MOV = '5' AND ROWNUM <= 10 GROUP BY USU_ORIGEN_MOV ORDER BY mis_averias DESC;
	  
	  INSERT INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Calcula averas]',parametrosIN,'OK',USER);
		
	  COMMIT;
	  -- Tratamiento de excepciones
	EXCEPTION
	WHEN OTHERS THEN -- handles all errors
	  ROLLBACK;
	  IF (error_desc IS NULL) THEN error_desc := SQLCODE; END IF;
	  INSERT   INTO INV_LOG(cod_log,fecha_log,funcion_log,param_in_log,param_out_log,usuario_log)
		VALUES(log_seq.nextval,SYSDATE,'([EST: Calcula averas]',parametrosIN,error_desc,USER);
	  COMMIT; 
	END EST_Calcula_averias;

END ESTADISTICAS_DWH_PQ;
