CREATE OR REPLACE PACKAGE INV_MANTENIMIENTOS_PQ AS
/******************************************************************************
   NOMBRE:   INV_MANTENIMIENTOS_PQ
   OBJECTIVO: ALTA, Y MODIFICACIN DE LAS TABLAS DEL ESQUEMA DE INVENTARIO
******************************************************************************/

		  /******************************************************************************
		    Descripcin: 		Alta de nuevo empleado en el sistema 
			Requisitos:			Rol administrador o gestor funcional  
			Parmetro entrada :
								*p_nif: nif/dni del empleado nuevo 
								*p_nombre: nombre del nuevo empleado 
								*p_apellidos: apellidos del nuevo empleado 
								*p_sexo: sexo del empleado 
								p_telefono: telfono de contacto del nuevo empleado 
								NO: p_es_supervisor:  indica si este usuario es el supervisor del departamento 
								*p_departamento: departamento al que se asigna el empleado
								*p_sede: sede a la que se asigna al empleado (NO ESTABA EN ESPECIFICACIN PAC2)
			Parmetro de salida :
								RSP 
								En caso de insercin con xito retorna el cdigo del identificador del registro 
			PRE-CONDICIN 
								El empleado no debe estar previamente de alta. Es decir, no debe existir otro empleado con el mismo DNI/NIF/NIE 
								Se deben completar todos los campos obligatorios (NOT NULL) 
								Existe previamente el departamento y la sede a la que se asigna el empleado. 
			POST-CONDICIN 		Se da de alta el empleado 
								Se registra accin en el log 
			Devuelve 
								Si todo ha ido bien: RSP = 'OK: Alta empleado con el Id = cod_empleado' 
								Mensajes de error: 
								'ERROR: hay campos obligatorios sin rellenar' 
								'ERROR: El empleado ya existe en el sistema' 
								'ERROR: No existe el departamento indicado' 
								'ERROR: No existe la sede indicada' 
								'ERROR: Formato de NIF/NIE incorrecto'  
		   ******************************************************************************/	
       
       
PROCEDURE INV_GEST_EMPLEADO_ALTA_PR  (
						    p_nif  IN INV_EMPLEADO.NIF_emp%TYPE,
						    p_nombre IN INV_EMPLEADO.nombre_emp%TYPE,
							p_apellidos IN INV_EMPLEADO.apellidos_emp%TYPE,
							p_sexo IN INV_EMPLEADO.sexo_emp%TYPE,
							p_telefono IN INV_EMPLEADO.telefono_emp%TYPE,
							p_fecha_nac IN INV_EMPLEADO.fecha_nac_emp%TYPE,
							p_email IN INV_EMPLEADO.email_emp%TYPE,
							p_cod_dep IN INV_DEPARTAMENTO.cod_dep%TYPE,
							p_cod_sede IN INV_SEDE.cod_sede%TYPE, -- AADIDO, NO ESTABA EN LA ESPECIFICACIN PAC2
							RSP OUT VARCHAR2,
							COD_NUEVO_EMPLEADO OUT INV_EMPLEADO.cod_emp%TYPE
							);
	

 /*
		    Descripcin Alta departamento
			Requisitos Rol administrador o gestor funcional
			Parmetro entrada
				*p_nombre: Nombre del departamento
				p_descripcin: Descripcin del departamento
				p_responsable: Identificador del empleado responsable del departamento
			Parmetro de salida
				OUT: RSP
				En caso de insercin con xito retorna el cdigo del identificador del
				registro
			PRE-CONDICIN No existe el departamento
				Se registra accin en la tabla LOG
				Devuelve Si todo ha ido bien:
				RSP = 'OK: Alta departamento con el Id = cod_departamento'
				Mensajes de error:
					ERROR: hay campos obligatorios sin rellenar
					ERROR: El departamento ya existe en el sistema
					ERROR: No existe ese empleado en el sistema
*/	


PROCEDURE INV_GEST_DEP_ALTA_PR  (p_nombre  IN INV_DEPARTAMENTO.nom_dep%TYPE,
              p_descripcion IN INV_DEPARTAMENTO.desc_dep%TYPE,
							p_responsable IN INV_EMPLEADO.cod_emp%TYPE,
							RSP OUT VARCHAR2,
							COD_NUEVO_DEPARTAMENTO OUT INV_DEPARTAMENTO.cod_dep%TYPE
							); 		 	



		  /******************************************************************************
			Descripcin Alta de nueva sede en el sistema
			Requisitos Rol administrador o gestor funcional
						La sede no debe estar previamente de alta.
						Se deben completar todos los campos obligatorios (NOT NULL)
						Existe previamente el departamento al que pertenece la sede.
			Parmetro entrada
						*p_nombre: Nombre con el que se designa la sede
						*P_pais: Pais donde est la sede
						*P_localidad: Localidad donde se ubica la sede
						*P_cod_postal: codigo postal al que corresponde la sede
						P_nom_via: Nombre de la calle donde se ubica la sede
						P_num_via: Nmero del portal
						P_piso: Piso donde se ubica la sede
						P_puerta: puerta donde se ubica la sede
			Parmetro de salida
				RSP
				En caso de insercin con xito retorna el cdigo del identificador de la
				sede
			PRE-CONDICIN No existe la sede
			POST-CONDICIN Se da de alta la nueva sede
							Se registra accin en la tabla LOG
							Devuelve Si todo ha ido bien:
							RSP = 'OK: Alta sede con el Id = cod_sede'
							Mensajes de error:
							ERROR: hay campos obligatorios sin rellenar
							ERROR: La sede ya existe en el sistema
							ERROR: No existe el departamento indicado
							ERROR: Formato de CP incorrecto
		   ******************************************************************************/		

PROCEDURE INV_GEST_SEDE_ALTA_PR  (
							p_nombre IN INV_SEDE.nom_sede%TYPE,
							p_pais IN INV_SEDE.pais_sede%TYPE,
							p_localidad IN INV_SEDE.localidad_sede%TYPE,
							p_cod_postal IN INV_SEDE.cod_postal_sede%TYPE,
							p_nom_via IN INV_SEDE.nom_via_sede%TYPE,
							p_num_via IN INV_SEDE.numero_via_sede%TYPE,
							p_piso IN INV_SEDE.piso_sede%TYPE,
							p_puerta IN INV_SEDE.puerta_sede%TYPE,
							p_departamento IN INV_DEPARTAMENTO.cod_dep%TYPE,
							RSP OUT VARCHAR2,
							COD_NUEVA_SEDE OUT INV_SEDE.cod_sede%TYPE
							); 

  /******************************************************************************
			Descripcin Alta de nuevo proveedor en el sistema
			Requisitos Rol administrador o responsable financiero
						El proveedor no debe estar previamente de alta.
						Se deben completar todos los campos obligatorios (NOT NULL)
			Parmetro entrada
						* p_CIF_provee: CIF del proveedor
						* p_Nombre_prov: Nombre del proveedor
						p_Direccin_prov
						p_Contacto_prov
						p_Email_prov
						p_Telefono_prov
						*p_departamento: Departamento al que pertenece la sede
			Parmetro de salida
						RSP
						En caso de insercin con xito retorna el cdigo del identificador del
						proveedor
			PRE-CONDICIN No existe el proveedor
			POST-CONDICIN Se da de alta el nuevo proveedor
						Se registra accin en la tabla LOG
						Devuelve Si todo ha ido bien:
						RSP = 'OK: Alta proveedor con el Id = cod_prov'
						Mensajes de error:
						ERROR: hay campos obligatorios sin rellenar
						ERROR: El proveedor ya existe en el sistema
		   ******************************************************************************/		
       
PROCEDURE INV_GEST_PROVEEDOR_ALTA_PR  (
						    p_CIF_provee  IN INV_PROVEEDOR.cif_prov%TYPE,
							p_Nombre_prov IN INV_PROVEEDOR.nombre_prov%TYPE,
							p_Direccin_prov IN INV_PROVEEDOR.direccion_prov%TYPE,
							p_Contacto_prov IN INV_PROVEEDOR.contacto_prov%TYPE,
							p_Email_prov IN INV_PROVEEDOR.email_prov%TYPE,
							p_Telefono_prov IN INV_PROVEEDOR.telefono_prov%TYPE,
							RSP OUT VARCHAR2,
							COD_NUEVO_PROVEEDOR OUT INV_PROVEEDOR.cod_prov%TYPE
							); 
		

  /******************************************************************************
			Descripcin Alta de producto
						Requisitos Rol administrador o gestor funcional o Informtico?
			Parmetro entrada
						*p_nombre : Nombre del producto
						*p_tipo_prod: Tipo del producto
						*p_marca_prod: Marca fabricante del producto
						p_descripcin: Descripcin del producto
			Parmetro de salida
						OUT: RSP
						En caso de insercin con xito retorna el cdigo del identificador del
						registro
			PRE-CONDICIN No existe el producto
			POST-CONDICIN Se da de alta el producto
						Se registra accin en la tabla LOG
						Devuelve Si todo ha ido bien:
							RSP = 'OK: Alta producto con el Id = cod_producto'
						Mensajes de error:
							'ERROR: hay campos obligatorios sin rellenar'
							'ERROR: El producto ya existe en el sistema'
							'ERROR: Tipo de producto no existente
							ERROR: Marca del producto no existente
		   ******************************************************************************/		
PROCEDURE INV_GEST_PRODUCTO_ALTA_PR  (p_nombre  IN INV_PRODUCTO.nombre_prod%TYPE,
                            p_tipo_prod IN INV_PRODUCTO.cod_tipo_prod%TYPE,
							p_marca_prod IN INV_PRODUCTO.cod_prod%TYPE,
							p_descripcion IN INV_PRODUCTO.desc_prod%TYPE,
							RSP OUT VARCHAR2,
							COD_NUEVO_PRODUCTO OUT INV_PRODUCTO.cod_prod%TYPE
							); 

/******BAJAS******/

/*Descripcin Baja de un empleado de la empresa 
Requisitos Rol administrador o gestor funcional 
            El empleado debe estar dado de alta previamente 
Parmetro entrada 
            *p_id_empleado: Identificador del empleado  
Parmetro de salida 
            RSP 
PRE-CONDICIN Existe el empleado 
POST-CONDICIN Se da de baja el empleado 
Devuelve 
  Si todo ha ido bien: RSP = 'OK: Baja empleado realizada correctamente'
  Mensajes de error: 
                  'ERROR: hay campos obligatorios sin rellenar' 
                  'ERROR: El empleado no existe en el sistema' */

PROCEDURE INV_GEST_EMPLEADO_BAJA_PR(
              p_id_empleado IN INV_EMPLEADO.cod_emp%TYPE,
							RSP OUT VARCHAR2
							); 
         
PROCEDURE INV_GEST_EMPLEADO_ELIMINA_PR(
              p_id_empleado IN INV_EMPLEADO.cod_emp%TYPE,
							RSP OUT VARCHAR2
							); 
           

						
--MODIFICACIONES DEPARTAMENTO
PROCEDURE INV_GEST_DEP_MODIF_PR  (
				p_cod_dep IN INV_DEPARTAMENTO.cod_dep%TYPE,
				p_nombre  IN INV_DEPARTAMENTO.nom_dep%TYPE,
				p_descripcion IN INV_DEPARTAMENTO.desc_dep%TYPE,
				p_responsable IN INV_EMPLEADO.cod_emp%TYPE,
				RSP OUT VARCHAR2
				); 	
				
				
		  /******************************************************************************
		    Descripcin: 		Modificacin empleado en el sistema 
			Requisitos:			Rol administrador o gestor funcional  
			Parmetro entrada :
								*p_cod_emp: codigo empleado
								p_nif: nif/dni del empleado nuevo 
								p_nombre: nombre del nuevo empleado 
								p_apellidos: apellidos del nuevo empleado 
								p_sexo: sexo del empleado 
								p_telefono: telfono de contacto del nuevo empleado 
								p_es_supervisor:  indica si este usuario es el supervisor del departamento 
								p_departamento: departamento al que se asigna el empleado
								p_sede: sede a la que se asigna al empleado (NO ESTABA EN ESPECIFICACIN PAC2)
			Parmetro de salida : RSP 
			PRE-CONDICIN 
								El empleado est previamente de alta.  
			POST-CONDICIN 		Se modifica el empleado 
								Se registra accin en el log 
			Devuelve 
								Si todo ha ido bien: RSP = 'OK: Empleado modificado' 
								Mensajes de error: 
								'ERROR: hay campos obligatorios sin rellenar' 
								'ERROR: El empleado no existe en el sistema' 
		   ******************************************************************************/	
       
       
PROCEDURE INV_GEST_EMPLEADO_MODIF_PR  (
							p_cod_emp IN INV_EMPLEADO.cod_emp%TYPE,
						    p_nif  IN INV_EMPLEADO.NIF_emp%TYPE,
						    p_nombre IN INV_EMPLEADO.nombre_emp%TYPE,
							p_apellidos IN INV_EMPLEADO.apellidos_emp%TYPE,
							p_sexo IN INV_EMPLEADO.sexo_emp%TYPE,
							p_telefono IN INV_EMPLEADO.telefono_emp%TYPE,
							p_fecha_nac IN INV_EMPLEADO.fecha_nac_emp%TYPE,
							p_email IN INV_EMPLEADO.email_emp%TYPE,
							p_cod_dep IN INV_DEPARTAMENTO.cod_dep%TYPE,
							p_cod_sede IN INV_SEDE.cod_sede%TYPE, 
							p_estado IN INV_EMPLEADO.estado_emp%TYPE,
							RSP OUT VARCHAR2
							);
         
END INV_MANTENIMIENTOS_PQ;
/


