
----------------------------------------
---------- PACKAGE BODY ----------------
----------------------------------------


CREATE OR REPLACE PACKAGE BODY	INV_MANTENIMIENTOS_PQ AS
   
-- Alta de nuevo empleado
PROCEDURE INV_GEST_EMPLEADO_ALTA_PR  (
						    p_nif  IN INV_EMPLEADO.NIF_emp%TYPE,
						    p_nombre IN INV_EMPLEADO.nombre_emp%TYPE,
							p_apellidos IN INV_EMPLEADO.apellidos_emp%TYPE,
							p_sexo IN INV_EMPLEADO.sexo_emp%TYPE,
							p_telefono IN INV_EMPLEADO.telefono_emp%TYPE,
							p_fecha_nac IN INV_EMPLEADO.fecha_nac_emp%TYPE,
							p_email IN INV_EMPLEADO.email_emp%TYPE,
							p_cod_dep IN INV_DEPARTAMENTO.cod_dep%TYPE,
							p_cod_sede IN INV_SEDE.cod_sede%TYPE, -- AADIDO, NO ESTABA EN LA ESPECIFICACIN PAC2
							RSP OUT VARCHAR2,
							COD_NUEVO_EMPLEADO OUT INV_EMPLEADO.cod_emp%TYPE
							) IS
  parametrosIN varchar2(1000);
  v_es_valido number;
  num_dep number;
  num_sede number;
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
BEGIN
	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_nif: '||p_nif||', p_nombre: '||p_nombre||', p_apellidos: '||p_apellidos||', p_sexo: '||p_sexo||', p_telefono: '||p_telefono||', p_fecha_nac: '||p_fecha_nac||', p_email: '||p_email||', p_cod_dep: '||p_cod_dep||', p_cod_sede: '||p_cod_sede;

 --Verifica que todos los campos obligatorios tienen valor
	IF (p_nif IS NULL OR p_nombre IS NULL OR p_apellidos IS NULL OR p_cod_dep IS NULL OR p_cod_sede IS NULL) THEN 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 RAISE e_error;
	END IF;

  -- Verifica que el NIF es correcto
  v_es_valido := AUXILIAR_PQ.NIF_valido(p_nif);
  IF (v_es_valido = 0) THEN
    error_desc:='ERROR: El NIF no es vlido, formato incorrecto';
    RAISE e_error;
  END IF;
  --Verifica que el departamento existe en la Base de Dades
  BEGIN
    SELECT 	count(cod_dep)
    INTO	num_dep
    FROM	inv_departamento
    WHERE cod_dep = p_cod_dep; 
  END;
  IF (num_dep = 0) THEN error_desc:='ERROR: El departamento indicado no existe'; RAISE e_error; END IF;
 
  --Verifica que la sede existe en la Base de Dades
  
  BEGIN
    SELECT 	count(cod_sede)
    INTO	num_sede
    FROM	inv_sede
    WHERE cod_sede = p_cod_sede; 
  END;
  IF (num_sede = 0) THEN error_desc:='ERROR: La sede indicada no existe'; RAISE e_error; END IF;
  
  -- Insertamos nuevo empleado
  INSERT INTO INV_EMPLEADO (cod_emp, NIF_emp, nombre_emp, apellidos_emp, telefono_emp, fecha_nac_emp, estado_emp, sexo_emp, email_emp, cod_dep_emp, cod_sede_emp)
  VALUES (empleado_seq.nextval,p_nif,p_nombre,p_apellidos,p_telefono,p_fecha_nac,'A',p_sexo,p_email,p_cod_dep,p_cod_sede);
  --COMMIT;
  COD_NUEVO_EMPLEADO := empleado_seq.currval;
  RSP := 'OK';
		
  -- Actualizamos el log con la accin realizada
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_ALTA_PR', parametrosIN, RSP,user);
  
  COMMIT; -- Confirmamos las operaciones
  
  -- Actualizamos estadsticas
  ESTADISTICAS_DWH_PQ.EST_Nuevo_empleado;
  
  -- Tratamiento de excepciones

EXCEPTION  
  WHEN DUP_VAL_ON_INDEX THEN
      ROLLBACK;
      error_desc := 'ERROR: Valor duplicado para PK';
      INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_ALTA_PR', parametrosIN, error_desc,user);
	WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SUBSTR(SQLERRM, 1, 100) ; END IF;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_ALTA_PR', parametrosIN, error_desc,user);
      COMMIT;
      
END INV_GEST_EMPLEADO_ALTA_PR; 
	


PROCEDURE INV_GEST_DEP_ALTA_PR  (p_nombre  IN INV_DEPARTAMENTO.nom_dep%TYPE,
							p_descripcion IN INV_DEPARTAMENTO.desc_dep%TYPE,
							p_responsable IN INV_EMPLEADO.cod_emp%TYPE,
							RSP OUT VARCHAR2,
							COD_NUEVO_DEPARTAMENTO OUT INV_DEPARTAMENTO.cod_dep%TYPE
							) IS
	parametrosIN varchar2(1000);
	num_emp	number;
	error_desc varchar2(1000); -- descripcion del error
	e_error exception; --error controlado
BEGIN
	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_nombre: '||p_nombre||', p_descripcion: '||p_descripcion||', p_responsable: '||p_responsable;
	
	 --Verifica que todos los campos obligatorios tienen valor
	IF (p_nombre IS NULL) THEN 
	 error_desc:='ERROR: Parmetro NOMBRE no puede ser nulo';
	 RAISE e_error;
	END IF;
	
	-- Verificamos que existe el responsable indicado. Como no es obligatorio: Si no se indica no debe dar error
	IF (p_responsable IS NOT NULL) THEN 	
		  BEGIN
			SELECT 	COUNT(cod_emp)
			INTO	num_emp
			FROM	inv_empleado
			WHERE 	cod_emp = p_responsable; 
		  END;
		  IF (num_emp = 0) THEN error_desc:='ERROR: El responsable indicado no existe'; RAISE e_error; END IF;
	END IF;
  
  
	INSERT INTO INV_DEPARTAMENTO(cod_dep,nom_dep,desc_dep,cod_responsable_dep)
			VALUES(departamento_seq.nextval, p_nombre,p_descripcion,p_responsable);
	
  COD_NUEVO_DEPARTAMENTO := departamento_seq.currval;
	RSP := 'OK';
	
	-- Actualizamos el log con la accin realizada
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta departamento]INV_MANTENIMIENTOS_PQ.INV_GEST_DEP_ALTA_PR', parametrosIN, RSP,user);
  COMMIT;
	
EXCEPTION  
  WHEN DUP_VAL_ON_INDEX THEN
      ROLLBACK;
      error_desc := 'ERROR: Valor duplicado para PK';
      INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta departamento]INV_MANTENIMIENTOS_PQ.INV_GEST_DEP_ALTA_PR', parametrosIN, error_desc,user);
      COMMIT;
	WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SUBSTR(SQLERRM, 1, 100) ; END IF;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta departamento]INV_MANTENIMIENTOS_PQ.INV_GEST_DEP_ALTA_PR', parametrosIN, error_desc,user);
      COMMIT;
								
END; -- END INV_GEST_DEP_ALTA_PR	


PROCEDURE INV_GEST_SEDE_ALTA_PR  (
							p_nombre IN INV_SEDE.nom_sede%TYPE,
							p_pais IN INV_SEDE.pais_sede%TYPE,
							p_localidad IN INV_SEDE.localidad_sede%TYPE,
							p_cod_postal IN INV_SEDE.cod_postal_sede%TYPE,
							p_nom_via IN INV_SEDE.nom_via_sede%TYPE,
							p_num_via IN INV_SEDE.numero_via_sede%TYPE,
							p_piso IN INV_SEDE.piso_sede%TYPE,
							p_puerta IN INV_SEDE.puerta_sede%TYPE,
							p_departamento IN INV_DEPARTAMENTO.cod_dep%TYPE,
							RSP OUT VARCHAR2,
							COD_NUEVA_SEDE OUT INV_SEDE.cod_sede%TYPE
							) IS
  num_sedes NUMBER;
  num_dep NUMBER;
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
	
BEGIN
  
  -- Concatenamos los parmetros de entrada para el LOG
  parametrosIN := 'p_nombre: '||p_nombre||', p_pais: '||p_pais||', p_localidad: '||p_localidad||', p_cod_postal: '||p_cod_postal||', p_nom_via: '||p_nom_via||', p_num_via: '||p_num_via||', p_piso: '||p_piso||', p_puerta: '||p_puerta||', p_departamento: '||p_departamento;
  
  
	 --Verifica que todos los campos obligatorios tienen valor
	IF (p_nombre IS NULL OR p_pais IS NULL OR p_cod_postal IS NULL OR p_departamento IS NULL) THEN 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 RAISE e_error;
	END IF;
	
  
  BEGIN
	 -- Comprobamos que el departamento exista
    SELECT 	COUNT(cod_dep)
    INTO	num_dep
    FROM	inv_departamento
    WHERE   cod_dep = p_departamento;
  END;
  
  IF (num_dep = 0) THEN error_desc:='ERROR: El departamento indicado no existe'; RAISE e_error; END IF;
  
	INSERT INTO INV_SEDE(cod_sede,nom_sede,pais_sede,localidad_sede,cod_postal_sede,nom_via_sede,numero_via_sede,piso_sede, puerta_sede,cod_dep_sede)
	VALUES(sede_seq.nextval, p_nombre,p_pais,p_localidad,p_cod_postal, p_nom_via,p_num_via,p_piso,p_puerta,p_departamento);

	RSP := 'OK';
	COD_NUEVA_SEDE := sede_seq.currval;	
  
	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Alta sede]INV_MANTENIMIENTOS_PQ.INV_GEST_SEDE_ALTA_PR', parametrosIN, RSP,user);
	COMMIT;
	  
EXCEPTION  
  WHEN DUP_VAL_ON_INDEX THEN
        ROLLBACK;
        error_desc := 'ERROR: Valor duplicado para PK';
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta sede]INV_MANTENIMIENTOS_PQ.INV_GEST_SEDE_ALTA_PR', parametrosIN, error_desc,user);
  COMMIT;
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SUBSTR(SQLERRM, 1, 100) ; END IF;
	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta sede]INV_MANTENIMIENTOS_PQ.INV_GEST_SEDE_ALTA_PR', parametrosIN, error_desc,user);
  COMMIT;
END INV_GEST_SEDE_ALTA_PR	;
	
	
	
PROCEDURE INV_GEST_PROVEEDOR_ALTA_PR  (
							p_CIF_provee  IN INV_PROVEEDOR.cif_prov%TYPE,
							p_Nombre_prov IN INV_PROVEEDOR.nombre_prov%TYPE,
							p_Direccin_prov IN INV_PROVEEDOR.direccion_prov%TYPE,
							p_Contacto_prov IN INV_PROVEEDOR.contacto_prov%TYPE,
							p_email_prov IN INV_PROVEEDOR.email_prov%TYPE,
							p_Telefono_prov IN INV_PROVEEDOR.telefono_prov%TYPE,
							RSP OUT VARCHAR2,
							COD_NUEVO_PROVEEDOR OUT INV_PROVEEDOR.cod_prov%TYPE
							) IS
	error_desc varchar2(1000); -- descripcion del error
	e_error exception; --error controlado
    parametrosIN varchar2(1000);
BEGIN
	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_CIF_provee: '||p_CIF_provee||', p_Nombre_prov: '||p_Nombre_prov||', p_Direccin_prov: '||p_Direccin_prov||', p_Contacto_prov: '||p_Contacto_prov||', p_email_prov: '||p_email_prov||', p_Telefono_prov: '||p_Telefono_prov;
	
	 --Verifica que todos los campos obligatorios tienen valor
	if (p_CIF_provee is NULL or p_Nombre_prov is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

	
		INSERT INTO INV_PROVEEDOR(cod_prov,CIF_prov,nombre_prov,direccion_prov,contacto_prov,email_prov,telefono_prov)
				VALUES(proveedor_seq.nextval, p_CIF_provee,p_Nombre_prov,p_Direccin_prov,p_Contacto_prov, p_Email_prov,p_Telefono_prov);
		RSP := 'OK';
		COD_NUEVO_PROVEEDOR := proveedor_seq.currval;	

		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Alta Proveedor]INV_MANTENIMIENTOS_PQ.INV_GEST_PROVEEDOR_ALTA_PR', parametrosIN, RSP,user);
		COMMIT;
	
EXCEPTION  
  WHEN DUP_VAL_ON_INDEX THEN
          ROLLBACK;
          error_desc := 'ERROR: Valor duplicado para PK';
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Alta Proveedor]INV_MANTENIMIENTOS_PQ.INV_GEST_PROVEEDOR_ALTA_PR', parametrosIN, error_desc,user);
    COMMIT;
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SUBSTR(SQLERRM, 1, 100) ; END IF;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta Proveedor]INV_MANTENIMIENTOS_PQ.INV_GEST_PROVEEDOR_ALTA_PR', parametrosIN, error_desc,user);
  COMMIT;
END; --INV_GEST_PROVEEDOR_ALTA_PR
	
	
PROCEDURE INV_GEST_PRODUCTO_ALTA_PR  (p_nombre  IN INV_PRODUCTO.nombre_prod%TYPE,
                            p_tipo_prod IN INV_PRODUCTO.cod_tipo_prod%TYPE,
							p_marca_prod IN INV_PRODUCTO.cod_prod%TYPE,
							p_descripcion IN INV_PRODUCTO.desc_prod%TYPE,
							RSP OUT VARCHAR2,
							COD_NUEVO_PRODUCTO OUT INV_PRODUCTO.cod_prod%TYPE
							) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_tipo number;
  num_marca number;
BEGIN

	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_nombre: '||p_nombre||', p_tipo_prod: '||p_tipo_prod||'p_marca_prod: '||p_marca_prod||', p_descripcion: '||p_descripcion;
	
 --Verifica que todos los campos obligatorios tienen valor
	if (p_nombre is NULL or p_tipo_prod is NULL or p_marca_prod is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;
	
	--Verifica que el tipo de producto existe en la Base de Dades
  BEGIN
    select 	count(cod_tipo_prod)
    into	num_tipo
    from	inv_tipo_producto
    where 	cod_tipo_prod = p_tipo_prod; 
  END;
  if (num_tipo = 0) then error_desc:='ERROR: El tipo de producto indicado no existe'; raise e_error; end if;
  
  	--Verifica que la marca de producto existe en la Base de Dades
  BEGIN
    SELECT 	count(cod_marca)
    into	num_marca
    from	inv_marca
    where 	cod_marca = p_marca_prod; 
  END;
  IF (num_marca = 0) THEN error_desc:='ERROR: La marca del producto indicado no existe'; RAISE e_error; END IF;
	
	INSERT INTO INV_PRODUCTO(cod_prod,nombre_prod,cod_tipo_prod,cod_marca_prod,desc_prod)
			VALUES(producto_seq.nextval, p_nombre,p_tipo_prod,p_marca_prod,p_descripcion);

	RSP := 'OK';
	COD_NUEVO_PRODUCTO := producto_seq.currval;	
		
	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Alta producto]INV_MANTENIMIENTOS_PQ.INV_GEST_PRODUCTO_ALTA_PR', parametrosIN, RSP,user);
	COMMIT;
		
EXCEPTION  
  WHEN DUP_VAL_ON_INDEX THEN
          ROLLBACK;
          error_desc := 'ERROR: Valor duplicado para PK';
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta producto]INV_MANTENIMIENTOS_PQ.INV_GEST_PRODUCTO_ALTA_PR', parametrosIN, error_desc,user);
  COMMIT;
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SUBSTR(SQLERRM, 1, 100) ; END IF;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta producto]INV_MANTENIMIENTOS_PQ.INV_GEST_PRODUCTO_ALTA_PR', parametrosIN, error_desc,user);
  COMMIT;
	  
END INV_GEST_PRODUCTO_ALTA_PR;


PROCEDURE INV_GEST_EMPLEADO_BAJA_PR(
              p_id_empleado IN INV_EMPLEADO.cod_emp%TYPE,
							RSP OUT VARCHAR2
							) IS       
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_emp number;
  
BEGIN
  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_id_empleado: '||p_id_empleado;
  
 
  --Verifica que existe el empleado
  BEGIN
    SELECT 	count(cod_emp)
    INTO	num_emp
    FROM	INV_EMPLEADO
    WHERE 	cod_emp = p_id_empleado; 
  END;
  IF (num_emp = 0) THEN error_desc:='ERROR: El empleado no existe o no est de baja'; RAISE e_error; END IF;
  
  UPDATE INV_EMPLEADO SET estado_emp = 'B' WHERE cod_emp = p_id_empleado;

	RSP := 'OK';
  
	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_BAJA_PR', parametrosIN, RSP,user);
	
	COMMIT;
    -- Actualizamos estadsticas
  ESTADISTICAS_DWH_PQ.EST_Baja_empleado;
	
EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SQLCODE ||SUBSTR(SQLERRM, 1, 100) ; END IF;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_BAJA_PR', parametrosIN, error_desc,user);
  COMMIT;  
END INV_GEST_EMPLEADO_BAJA_PR;


-- Pre: el empleado debe estar de baja y sin dispositivos asignados!!!!
PROCEDURE INV_GEST_EMPLEADO_ELIMINA_PR(
              p_id_empleado IN INV_EMPLEADO.cod_emp%TYPE,
							RSP OUT VARCHAR2) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_emp number;
BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_id_empleado: '||p_id_empleado;
    --Verifica que existe el empleado y que est de baja
  BEGIN
    SELECT 	count(cod_emp)
    INTO	num_emp
    FROM	INV_EMPLEADO
    WHERE 	cod_emp = p_id_empleado AND estado_emp='B'; 
  END;
  IF (num_emp = 0) THEN error_desc:='ERROR: El empleado no existe o no est de baja'; RAISE e_error; END IF;
  
  
  BEGIN
    DELETE FROM INV_EMPLEADO WHERE cod_emp = p_id_empleado;
  END;
  RSP := 'OK';
  
	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Elimina empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_ELIMINA_PR', parametrosIN, RSP,user);

	COMMIT;
	
EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SUBSTR(SQLERRM, 1, 100) ; END IF;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Elimina empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_ELIMINA_PR', parametrosIN, error_desc,user);
  COMMIT; 
END INV_GEST_EMPLEADO_ELIMINA_PR;

  
--MODIFICACIONES/MANTENIMIENTO--

PROCEDURE INV_GEST_DEP_MODIF_PR  (
				p_cod_dep IN INV_DEPARTAMENTO.cod_dep%TYPE,
				p_nombre  IN INV_DEPARTAMENTO.nom_dep%TYPE,
				p_descripcion IN INV_DEPARTAMENTO.desc_dep%TYPE,
				p_responsable IN INV_EMPLEADO.cod_emp%TYPE,
				RSP OUT VARCHAR2) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  v_num_dep number;
  v_num_emp number;
BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_dep: '||p_cod_dep||', p_nombre: '||p_nombre||', p_descripcion: '||p_descripcion||', p_responsable: '||p_responsable;
  
  --Verifica que existe el departamento
  BEGIN
    SELECT 	COUNT(cod_dep)
    INTO	v_num_dep
    FROM	INV_DEPARTAMENTO
    WHERE 	cod_dep = p_cod_dep;
  END;
  IF (v_num_dep = 0) THEN error_desc:='ERROR: El departamento indicado no existe'; RAISE e_error; END IF;
  
    --Verifica que responsable (si se desea modificar)
  IF (p_responsable IS NOT NULL) THEN
  BEGIN
    SELECT 	COUNT(cod_emp)
    INTO	v_num_emp
    FROM	INV_EMPLEADO
    WHERE 	cod_emp = p_responsable;
  END;
  IF (v_num_emp = 0) THEN error_desc:='ERROR: El empleado indicado no existe'; RAISE e_error; END IF;
  END IF;
  
  UPDATE INV_DEPARTAMENTO 
    SET nom_dep = NVL(p_nombre,nom_dep),
        desc_dep = NVL(p_descripcion,desc_dep),
        cod_responsable_dep = NVL(p_responsable,cod_responsable_dep)
    WHERE cod_dep = p_cod_dep;

  RSP:= 'OK';
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Mod depart]INV_MANTENIMIENTOS_PQ.INV_GEST_DEP_MODIF_PR', parametrosIN, RSP,user);
	
	COMMIT;
	
EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SUBSTR(SQLERRM, 1, 100) ; END IF;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Mod depart]INV_MANTENIMIENTOS_PQ.INV_GEST_DEP_MODIF_PR', parametrosIN, error_desc,user);
  COMMIT;
END INV_GEST_DEP_MODIF_PR;



PROCEDURE INV_GEST_EMPLEADO_MODIF_PR  (
							p_cod_emp IN INV_EMPLEADO.cod_emp%TYPE,
						    p_nif  IN INV_EMPLEADO.NIF_emp%TYPE,
						    p_nombre IN INV_EMPLEADO.nombre_emp%TYPE,
							p_apellidos IN INV_EMPLEADO.apellidos_emp%TYPE,
							p_sexo IN INV_EMPLEADO.sexo_emp%TYPE,
							p_telefono IN INV_EMPLEADO.telefono_emp%TYPE,
							p_fecha_nac IN INV_EMPLEADO.fecha_nac_emp%TYPE,
							p_email IN INV_EMPLEADO.email_emp%TYPE,
							p_cod_dep IN INV_DEPARTAMENTO.cod_dep%TYPE,
							p_cod_sede IN INV_SEDE.cod_sede%TYPE, -- AADIDO, NO ESTABA EN LA ESPECIFICACIN PAC2
							p_estado IN INV_EMPLEADO.estado_emp%TYPE,
							RSP OUT VARCHAR2
							) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  v_num_emp number;
BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_emp: '||p_cod_emp||'p_nif: '||p_nif||', p_nombre: '||p_nombre||', p_apellidos: '||p_apellidos||', p_sexo: '||p_sexo||'p_telefono: '||p_telefono||'p_fecha_nac: '||p_fecha_nac||', p_email: '||p_email||', p_cod_dep: '||p_cod_dep||', p_cod_sede: '||p_cod_sede;
	
  -- El empleado ha de existir
  BEGIN
    SELECT 	COUNT(cod_emp)
    INTO	v_num_emp
    FROM	INV_EMPLEADO
    WHERE 	cod_emp = p_cod_emp; 
  END;
  IF (v_num_emp = 0) THEN error_desc:='ERROR: El empleado no existe'; RAISE e_error; END IF;
  
  
  UPDATE INV_EMPLEADO
    SET NIF_emp = NVL(p_nif,NIF_emp),
        nombre_emp = NVL(p_nombre,nombre_emp),
        apellidos_emp = NVL(p_apellidos,apellidos_emp),
        telefono_emp = NVL(p_telefono,telefono_emp),
        fecha_nac_emp = NVL(p_fecha_nac,fecha_nac_emp),
        sexo_emp = NVL(p_sexo,sexo_emp),
        email_emp = NVL(p_email,email_emp),
        estado_emp = NVL(p_estado,estado_emp),
        cod_dep_emp = NVL(p_cod_dep,cod_dep_emp),
        cod_sede_emp = NVL(p_cod_sede,cod_sede_emp)
    WHERE cod_emp = p_cod_emp;

  RSP:= 'OK';
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Mod empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_MODIF_PR', parametrosIN, RSP,user);
	
	COMMIT;
	
EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  IF (error_desc IS NULL) THEN error_desc := 'ERROR: '||SUBSTR(SQLERRM, 1, 100) ; END IF;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Mod empleado]INV_MANTENIMIENTOS_PQ.INV_GEST_EMPLEADO_MODIF_PR', parametrosIN, error_desc,user);
  COMMIT;
END;
							
							
							
END INV_MANTENIMIENTOS_PQ; --END PACKAGE BODY
/