
----------------------------------------
---------- PACKAGE SPECIFICATION ----------------
----------------------------------------


CREATE OR REPLACE PACKAGE INV_PEDIDOS_PQ AS
   
/*Descripcin:  	Nuevo pedido
Requisitos: 	Rol administrador o gestor funcional
Parmetro entrada:
				*p_fecha_pedido: Fecha realizacin del pedido
				p_Autorizado
				p_Resp_financiero
				p_Fecha_ok_financiero
				p_Importe_pedido
				p_Tipo_gasto: Especificacin del tipo de gasto (gasto o inversin)
				p_Fecha envo
				p_Fecha recepcin
				P_Provee_pedido
Parmetro de salida
				RSP
				En caso de insercin con xito retorna el cdigo del pedido
PRE-CONDICIN 
				No existe el pedido
POST-CONDICIN 
				Se da de alta el pedido
				Se registra accin en la tabla LOG
				Devuelve Si todo ha ido bien:
					RSP = 'OK: Alta pedido con el Id = cod_pedido
				Mensajes de error:
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: El pedido ya existe en el sistema' */
				

PROCEDURE INV_GEST_PED_ALTA_PR (
				p_fec_realizacion IN INV_PEDIDO.fec_realizacion_ped%TYPE,
				p_resp_autorizar IN INV_PEDIDO.resp_autorizar_ped%TYPE,
				p_comentario IN INV_PEDIDO.motivo_denega_ped%TYPE,
				RSP OUT varchar2,
				COD_NUEVO_PEDIDO out INV_PEDIDO.cod_pedido%TYPE
);
				
	/*Descripcin: 		Elimina el pedido
	Requisitos:		 	Rol administrador o gestor funcional
	Parmetro entrada:
						*p_cod_pedido: codigo del pedido
	Parmetro de salida:
						RSP
						En caso de insercin con xito retorna el cdigo del pedido
						PRE-CONDICIN No existe el pedido
						POST-CONDICIN Se da de baja el pedido
						Se registra accin en la tabla LOG
						Devuelve 
							Si todo ha ido bien:
								RSP = 'OK: Baja pedido con el Id = cod_pedido
							Mensajes de error: 'ERROR: hay campos obligatorios sin rellenar'*/
       
PROCEDURE INV_GEST_PED_BAJA_PR (p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE, RSP OUT VARCHAR2);
    
    /*Descripcin Solicita peticin de un producto existente a un pedido existente
		Requisitos Rol administrador o gestor funcional
		Parmetro entrada
			p_cod_pedido: Cdigo del pedido
			p_cantidad: Cantidad de productos solicitados
			p_cod_producto: cdigo del producto solicitado
		Parmetro de salida
			OUT: RSP	
			En caso de insercin con xito retorna el cdigo del identificador del
			pedido
		PRE-CONDICIN No existe el pedido
		POST-CONDICIN Se da de alta el pedido
						Se registra accin en la tabla LOG
		Devuelve 
				Si todo ha ido bien:
					RSP = 'OK: Alta lnea pedido con el Id = cod_linea_pedido
				Mensajes de error:
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: El cdigo de pedido no existe en el sistema'
				'ERROR: El cdigo de producto no existe en el sistema'
				'ERROR: Cantidad no vlida'*/

PROCEDURE INV_GEST_PED_SOLICITA_PROD_PR (
	p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
	p_cantidad IN number,
	p_cod_producto IN INV_PRODUCTO.cod_prod%TYPE,
	RSP OUT VARCHAR2);
	
	
	/*Descripcin:	 	Solicita un nuevo producto para pedido existente
	Requisitos: 		Rol administrador o gestor funcional
	Parmetro entrada
						p_cod_pedido: Cdigo del pedido
						p_cantidad: Cantidad de productos solicitados
						-- parmetros correspondientes a un nuevo producto
						-- ver procedimiento: INV_GEST_PROD_ALTA_PR
						*p_nombre : Nombre del producto
						*p_tipo_prod: Tipo del producto
						*p_marca_prod: Marca fabricante del producto
						p_descripcin: Descripcin del producto
	Parmetro de salida:
						RSP
						En caso de insercin con xito retorna el cdigo del identificador del
						pedido
	PRE-CONDICIN 		El pedio existe previamente
	POST-CONDICIN 		Se da de alta el pedido
						Se registra accin en la tabla LOG
						Devuelve Si todo ha ido bien:
						RSP = 'OK: Alta lnea pedido con el Id = cod_linea_pedido
						Mensajes de error:
						'ERROR: hay campos obligatorios sin rellenar'
						'ERROR: El cdigo de pedido no existe en el sistema'
						'ERROR: El cdigo de producto no existe en el sistema'
						'ERROR: Cantidad no vlida'
						'ERROR: hay campos obligatorios sin rellenar'
						'ERROR: El producto ya existe en el sistema'
						'ERROR: Tipo de producto no existente
						ERROR: Marca del producto no existente */

/*
-- NO ES NECESARIO ESTE PROCEDIMIENTO
PROCEDURE INV_GEST_PED_NUEVO_PRODUCTO_PR (
						p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
						p_cantidad IN number,
						p_nombre IN INV_PRODUCTO.nombre_prod%TYPE,
						p_tipo_prod IN INV_TIPO_PRODUCTO.cod_tipo_prod%TYPE,
						p_marca_prod IN INV_MARCA.cod_marca%TYPE,
						p_descripcion IN INV_PRODUCTO.desc_prod%TYPE,
						RSP OUT varchar2); -- falta codigos de nuevo producto??? 
			*/			

/*Descripcin: Solicita un producto para un pedido
Requisitos: Rol administrador o gestor funcional
Parmetro entrada:
				p_cod_pedido: Cdigo del pedido
				p_cantidad: Cantidad de productos solicitados
				-- parmetros correspondientes a un nuevo producto
				-- ver procedimiento: INV_GEST_PROD_ALTA_PR
				*p_nombre : Nombre del producto
				*p_tipo_prod: Tipo del producto
				*p_marca_prod: Marca fabricante del producto
				p_descripcin: Descripcin del producto
Parmetro de salida:
				RSP
				En caso de insercin con xito retorna el cdigo del identificador del
				pedido
PRE-CONDICIN 	El pedio existe previamente
POST-CONDICIN 	Se da de alta el pedido
				Se registra accin en la tabla LOG
				Devuelve Si todo ha ido bien:
				RSP = 'OK: Alta lnea pedido con el Id = cod_linea_pedido
Mensajes de error:
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: El cdigo de pedido no existe en el sistema'
				'ERROR: El cdigo de producto no existe en el sistema'
				'ERROR: Cantidad no vlida'
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: El producto ya existe en el sistema'
				'ERROR: Tipo de producto no existente
				ERROR: Marca del producto no existente*/

/*
-- NO ES NECEARIO ESTE PROCEDIMIENTO
PROCEDURE INV_GEST_PED_SOLICITA_PROD_PR (
						p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
						p_cantidad IN number,
						p_nombre IN INV_PRODUCTO.nombre_prod%TYPE,
						p_tipo_prod IN INV_TIPO_PRODUCTO.cod_tipo_prod%TYPE,
						p_marca_prod IN INV_MARCA.cod_marca%TYPE,
						p_descripcion IN INV_PRODUCTO.desc_prod%TYPE,
						RSP OUT varchar2); -- falta codigos de nuevo producto??? 

				*/		
--FN25. EliminaLineaPedido
--FN26. ModificaLineaPedido

		/*Descripcin: 			Valida pedido
		Requisitos: 			Rol responsable departamento
		Parmetro entrada:
								*p_cod_pedido: Cdigo del pedido a validar
								autorizado: pedido autorizado o no
								Fecha_validacin: fecha de la decisin de validar o revocar el pedido
		Parmetro de salida:
								RSP
								En caso de insercin con xito retorna el cdigo del identificador del
								pedido
		PRE-CONDICIN: 			No existe el pedido
		POST-CONDICIN:			Se da de alta el pedido
								Se registra accin en la tabla LOG
								Devuelve Si todo ha ido bien:
								RSP = 'OK: Alta pedido con el Id = cod_pedido
								Mensajes de error:
								'ERROR: hay campos obligatorios sin rellenar'
								'ERROR: El pedido ya existe en el sistema'*/
								
PROCEDURE INV_GEST_PED_VALIDA_PR (
			p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
			p_autorizado IN INV_PEDIDO.esta_autorizado_ped%TYPE,
            p_fecha_valida IN DATE,
            p_motivo IN INV_PEDIDO.motivo_denega_ped%TYPE, -- !!!aadido: no estaba en la especificacin del documento original pac2
			RSP OUT varchar2);
						
						
	/*Descripcin:			 Aprueba la compra del pedido o la rechaza
	Requisitos:				 Rol responsable financiero
							 Uno o varios presupuestos presentados
	Parmetro entrada:
							*p_cod_pedido: Cdigo del pedido a validar
							Fecha_aprobacin: fecha de la decisin de validar o revocar el pedido
							P_cod_presupuesto: Presupuesto elegido
							P_motivo: Motivo de la denegacin de la peticin
	Parmetro de salida:
							RSP
							En caso de insercin con xito retorna el cdigo del identificador del
							pedido
	PRE-CONDICIN:			El pedido previamente aprobado por el responsable del departamento
							Uno o varios presupuestos presentados
	POST-CONDICIN:			Se aprueba o rechaza el pedido
							Se registra accin en la tabla LOG
							Devuelve Si todo ha ido bien:
							RSP = 'OK: Alta pedido con el Id = cod_pedido
							Mensajes de error:
							'ERROR: hay campos obligatorios sin rellenar'
							'ERROR: El pedido ya existe en el sistema'*/

-- El importe del pedido ser el del presupuesto escogido
PROCEDURE INV_GEST_PED_APRUEBA_COMPRA_PR (
						p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
						p_emp_responsable IN INV_PEDIDO.resp_autorizar_ped%TYPE,
						p_fecha_aprobacion IN DATE, --LO PONEMOS OBLIGATORIO PORQUE SINO NO SE PUEDEN SACAR ESTADSTICAS!!!
						p_cod_presupuesto IN INV_PRESUPUESTO.cod_presup%TYPE,
						p_motivo IN INV_PEDIDO.motivo_denega_ped%TYPE,
						p_tipo_gasto IN INV_PEDIDO.tipo_gasto_ped%TYPE, --EL FINANCIERO INDICA EL TIPO DE GASTO
						RSP OUT varchar2);
            

-- Pedido recibido: Necesario para estadsticas
PROCEDURE INV_GEST_PED_RECIBIDO_PR (
						p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
						p_fecha_recepcion IN DATE,
            RSP OUT varchar2);
						
						
	/*Descripcin:			 Alta de nuevo presupuesto
	Requisitos:				 Rol responsable financiero
	Parmetro entrada:
							*p_Fecha_presup
							*P_Validez_presup
							*P_Cod_proveedor_presup
							*P_Cod_pedido_presup
							P_Fecha_entrega
	Parmetro de salida
							RSP
							En caso de insercin con xito retorna el cdigo del identificador del
							presupuesto
	PRE-CONDICIN:			Existe un proveedor y un pedido
	POST-CONDICIN:		 	Se da de alta un presupuesto
							Se registra accin en la tabla LOG
							Devuelve Si todo ha ido bien:
							RSP = 'OK: Alta presupuesto con el Id = cod_ presupuesto
							Mensajes de error:
							'ERROR: hay campos obligatorios sin rellenar'*/

PROCEDURE INV_GEST_PED_ALTA_PRESUP_PR(
						p_fecha_presup IN DATE,
						p_validez_presup IN DATE,
						importe_presup NUMBER,
						p_cod_proveedor_presup IN INV_PROVEEDOR.cod_prov%TYPE,
						p_cod_pedido_presup IN INV_PEDIDO.cod_pedido%TYPE,
						RSP OUT varchar2,
						COD_NUEVO_PRESUP OUT number);

END INV_PEDIDOS_PQ; 
/