
----------------------------------------
---------- PACKAGE SPECIFICATION -------
---------- PEDIDOS ---------------------
----------------------------------------


CREATE OR REPLACE PACKAGE BODY INV_PEDIDOS_PQ AS
   
/*Descripcin:  	Nuevo pedido
Requisitos: 	Rol administrador o gestor funcional
Parmetro entrada:
				*p_fec_realizacion: Fecha realizacin del pedido
				p_resp_autorizar: Resp_autoriza
				p_comentario
Parmetro de salida
				RSP
				En caso de insercin con xito retorna el cdigo del pedido
PRE-CONDICIN 
				No existe el pedido
POST-CONDICIN 
				Se da de alta el pedido
				Se registra accin en la tabla LOG
				Devuelve Si todo ha ido bien:
					RSP = 'OK: Alta pedido con el Id = cod_pedido
				Mensajes de error:
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: El pedido ya existe en el sistema' */
				

PROCEDURE INV_GEST_PED_ALTA_PR (
				p_fec_realizacion IN INV_PEDIDO.fec_realizacion_ped%TYPE,
				p_resp_autorizar IN INV_PEDIDO.resp_autorizar_ped%TYPE,
				p_comentario IN INV_PEDIDO.motivo_denega_ped%TYPE,
				RSP OUT varchar2,
				COD_NUEVO_PEDIDO out INV_PEDIDO.cod_pedido%TYPE
				) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  v_num_emp number;
BEGIN

  -- Concatenamos los parametros de entrada para el LOG
  parametrosIN := 'p_fec_realizacion: '||p_fec_realizacion||', p_resp_autorizar: '||p_resp_autorizar;
  -- VALIDACIONES
   -- Valida que existe el proveedor
   BEGIN
    select 	count(cod_emp)
    into	v_num_emp
    from	INV_EMPLEADO
    where cod_emp = p_resp_autorizar; 
  END;
  if (v_num_emp = 0) then error_desc:='ERROR: El empleado indicado no existe'; raise e_error; end if;
  
  
  INSERT INTO inv_pedido (cod_pedido,fec_realizacion_ped, resp_autorizar_ped,esta_autorizado_ped,motivo_denega_ped)
  VALUES(pedido_seq.nextval, p_fec_realizacion, p_resp_autorizar,'0',p_comentario);
  RSP := 'OK';
  COD_NUEVO_PEDIDO := pedido_seq.currval;
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'[Alta Pedido]INV_PEDIDOS_PQ.INV_GEST_PEDIDO_ALTA_PR', parametrosIN, RSP,user);
  COMMIT;
  
EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  if (error_desc is NULL) then error_desc := SQLCODE; end if;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'[Alta Pedido]INV_PEDIDOS_PQ.INV_GEST_PEDIDO_ALTA_PR', parametrosIN, error_desc,user);
  COMMIT; 
END INV_GEST_PED_ALTA_PR;
				
	/*Descripcin: 		Elimina el pedido
	Requisitos:		 	Rol administrador o gestor funcional
	Parmetro entrada:
						*p_cod_pedido: codigo del pedido
	Parmetro de salida:
						RSP
						En caso de insercin con xito retorna el cdigo del pedido
						PRE-CONDICIN No existe el pedido
						POST-CONDICIN Se da de baja el pedido
						Se registra accin en la tabla LOG
						Devuelve 
							Si todo ha ido bien:
								RSP = 'OK: Baja pedido con el Id = cod_pedido
							Mensajes de error: 'ERROR: hay campos obligatorios sin rellenar'*/
       
PROCEDURE INV_GEST_PED_BAJA_PR (p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE, RSP OUT VARCHAR2) IS
	  error_desc varchar2(1000); -- descripcion del error
	  e_error exception; --error controlado
	  parametrosIN varchar2(1000);
	BEGIN
	  -- Concatenamos los parametros de entrada para el LOG
		parametrosIN := 'p_cod_pedido: '||p_cod_pedido;
		BEGIN
		DELETE FROM INV_PEDIDO WHERE cod_pedido = p_cod_pedido;
	  END;
		RSP := 'OK';
	  
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Baja pedido]INV_PEDIDOS_PQ.INV_GEST_PED_BAJA_PR', parametrosIN, RSP,user);
		
		COMMIT;
		
	EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
	  if (error_desc is NULL) then error_desc := SQLCODE; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Baja pedido]INV_PEDIDOS_PQ.INV_GEST_PED_BAJA_PR', parametrosIN, error_desc,user);
	  COMMIT;  
END INV_GEST_PED_BAJA_PR;



		/*Descripcin Solicita peticin de un producto existente a un pedido existente
		Requisitos Rol administrador o gestor funcional
		Parmetro entrada
			p_cod_pedido: Cdigo del pedido
			p_cantidad: Cantidad de productos solicitados
			p_cod_producto: cdigo del producto solicitado
		Parmetro de salida
			OUT: RSP	
			En caso de insercin con xito retorna el cdigo del identificador del
			pedido
		PRE-CONDICIN No existe el pedido
		POST-CONDICIN Se da de alta el pedido
						Se registra accin en la tabla LOG
		Devuelve 
				Si todo ha ido bien:
					RSP = 'OK: Alta lnea pedido con el Id = cod_linea_pedido
				Mensajes de error:
				'ERROR: hay campos obligatorios sin rellenar'
				'ERROR: El cdigo de pedido no existe en el sistema'
				'ERROR: El cdigo de producto no existe en el sistema'
				'ERROR: Cantidad no vlida'*/

PROCEDURE INV_GEST_PED_SOLICITA_PROD_PR (
	p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
	p_cantidad IN number,
	p_cod_producto IN INV_PRODUCTO.cod_prod%TYPE,
	RSP OUT VARCHAR2) IS
  
	parametrosIN varchar2(1000);
	error_desc varchar2(1000); -- descripcion del error
	e_error exception; --error controlado
	num_ped number;
	num_prod number;
BEGIN
-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_pedido: '||p_cod_pedido||', p_cantidad: '||p_cantidad||', p_cod_producto: '||p_cod_producto;
  
 --Verifica que todos los campos obligatorios tienen valor
	IF (p_cod_pedido IS NULL OR p_cantidad IS NULL OR p_cod_producto IS NULL) THEN 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 RAISE e_error;
	END IF;

-- Verifica que existe el pedido
  BEGIN
    SELECT 	count(cod_pedido)
    INTO	num_ped
    FROM	inv_pedido
    WHERE cod_pedido = p_cod_pedido; 
  END;
  IF (num_ped = 0) THEN error_desc:='ERROR: El pedido indicado no existe'; RAISE e_error; END IF;
  
  -- Verifica que existe el producto
  BEGIN
    SELECT 	COUNT(cod_prod)
    INTO	num_prod
    FROM	INV_PRODUCTO
    WHERE cod_prod = p_cod_producto; 
  END;
  IF (num_prod = 0) THEN error_desc:='ERROR: El producto indicado no existe'; RAISE e_error; END IF;
  
    -- Aadimos linea de pedido
  INSERT INTO INV_LINEA_PEDIDO (cod_lineap, cantidad_lineap, cod_producto_lineap, cod_pedido_lineap)
  VALUES (linea_pedido_seq.nextval,p_cantidad,p_cod_producto,p_cod_pedido);

  RSP := 'OK'; --Actualizacion de linea de pedido
  -- Actualizamos el log con la accin realizada
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta linea pedido]INV_PEDIDOS_PQ.INV_GEST_PED_SOLICITA_PROD_PR', parametrosIN, RSP,user);
  
  COMMIT; -- Confirmamos las operaciones
  
  -- Tratamiento de excepciones

EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta linea pedido]INV_PEDIDOS_PQ.INV_GEST_PED_SOLICITA_PROD_PR', parametrosIN, error_desc,user);
      COMMIT;
END INV_GEST_PED_SOLICITA_PROD_PR;
	


PROCEDURE INV_GEST_PED_SOLICITA_PROD_PR (
						p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
						p_cantidad IN number,
						p_nombre IN INV_PRODUCTO.nombre_prod%TYPE,
						p_tipo_prod IN INV_TIPO_PRODUCTO.cod_tipo_prod%TYPE,
						p_marca_prod IN INV_MARCA.cod_marca%TYPE,
						p_descripcion IN INV_PRODUCTO.desc_prod%TYPE,
						RSP OUT varchar2) IS -- falta codigos de nuevo producto??? 
BEGIN
NULL;
END INV_GEST_PED_SOLICITA_PROD_PR;
						
--FN25. EliminaLineaPedido
--FN26. ModificaLineaPedido

		/*Descripcin: 			Valida pedido
		Requisitos: 			Rol responsable departamento
		Parmetro entrada:
								*p_cod_pedido: Cdigo del pedido a validar
								autorizado: pedido autorizado o no
								Fecha_validacin: fecha de la decisin de validar o revocar el pedido
		Parmetro de salida:
								RSP
								En caso de insercin con xito retorna el cdigo del identificador del
								pedido
		PRE-CONDICIN: 			No existe el pedido
		POST-CONDICIN:			Se da de alta el pedido
								Se registra accin en la tabla LOG
								Devuelve Si todo ha ido bien:
								RSP = 'OK: Alta pedido con el Id = cod_pedido
								Mensajes de error:
								'ERROR: hay campos obligatorios sin rellenar'
								'ERROR: El pedido ya existe en el sistema'*/
								
PROCEDURE INV_GEST_PED_VALIDA_PR(
						p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
						p_autorizado IN INV_PEDIDO.esta_autorizado_ped%TYPE,
            p_fecha_valida IN DATE,
            p_motivo IN INV_PEDIDO.motivo_denega_ped%TYPE,
						RSP OUT varchar2) IS
  num_ped number;
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
BEGIN

-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_pedido: '||p_cod_pedido||', p_autorizado: '||p_autorizado||', p_fecha_valida: '||p_fecha_valida||', p_motivo:'||p_motivo;
  
 --Verifica que todos los campos obligatorios tienen valor
	if (p_cod_pedido is NULL or p_autorizado is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

  BEGIN
    select 	count(cod_pedido)
    into	num_ped
    from	INV_PEDIDO
    where cod_pedido = p_cod_pedido; 
  END;
  if (num_ped = 0) then error_desc:='ERROR: El numero de pedido indicado no existe'; raise e_error; end if;
  
  
  update INV_PEDIDO set esta_autorizado_ped = p_autorizado, fec_validacion_ped=p_fecha_valida
  where cod_pedido = p_cod_pedido;
  
  if (p_motivo is not null) then
    update INV_PEDIDO set motivo_denega_ped = p_motivo where cod_pedido=p_cod_pedido;
  end if;
  
  if (p_autorizado = 1) then
    RSP := 'OK: Responsable aprueba pedido'; --Actualizacion de linea de pedido
  elsif (p_autorizado = 0) then
    RSP := 'OK: Responsable rechaza pedido';
  else
    error_desc:='ERROR: No se ha indicado correctamente la autorizacin (1:SI,0:NO)'; raise e_error;
  end if;
  
  -- Actualizamos el log con la accin realizada
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Valida pedido]INV_PEDIDOS_PQ.INV_GEST_PED_VALIDA_PR', parametrosIN, RSP,user);
  
  COMMIT; -- Confirmamos las operaciones
  
  -- Tratamiento de excepciones
EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Valida pedido]INV_PEDIDOS_PQ.INV_GEST_PED_VALIDA_PR', parametrosIN, error_desc,user);
      COMMIT;
END INV_GEST_PED_VALIDA_PR;
						
						
	/*Descripcin:			 Aprueba la compra del pedido o la rechaza
	Requisitos:				 Rol responsable financiero
							 Uno o varios presupuestos presentados
	Parmetro entrada:
							*p_cod_pedido: Cdigo del pedido a validar
							Fecha_aprobacin: fecha de la decisin de validar o revocar el pedido
							P_cod_presupuesto: Presupuesto elegido
							P_motivo: Motivo de la denegacin de la peticin
	Parmetro de salida:
							RSP
							En caso de insercin con xito retorna el cdigo del identificador del
							pedido
	PRE-CONDICIN:			El pedido previamente aprobado por el responsable del departamento
							Uno o varios presupuestos presentados
	POST-CONDICIN:			Se aprueba o rechaza el pedido
							Se registra accin en la tabla LOG
							Devuelve Si todo ha ido bien:
							RSP = 'OK: Alta pedido con el Id = cod_pedido
							Mensajes de error:
							'ERROR: hay campos obligatorios sin rellenar'
							'ERROR: El pedido ya existe en el sistema'*/

PROCEDURE INV_GEST_PED_APRUEBA_COMPRA_PR (
						p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
						p_emp_responsable IN INV_PEDIDO.resp_autorizar_ped%TYPE,
						p_fecha_aprobacion IN DATE, --LO PONEMOS OBLIGATORIO PORQUE SINO NO SE PUEDEN SACAR ESTADSTICAS!!!
						p_cod_presupuesto IN INV_PRESUPUESTO.cod_presup%TYPE,
						p_motivo IN INV_PEDIDO.motivo_denega_ped%TYPE,
						p_tipo_gasto IN INV_PEDIDO.tipo_gasto_ped%TYPE, --EL FINANCIERO INDICA EL TIPO DE GASTO, NO EL AUXILIAR!!!
						RSP OUT varchar2) IS
  num_ped number;
  num_presup number;
  num_emp number;
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  importe_ganador number(10,2);
  v_cod_proveedor number;
BEGIN

-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_pedido: '||p_cod_pedido||', p_fecha_aprobacion: '||p_fecha_aprobacion||', p_cod_presupuesto: '||p_cod_presupuesto||', p_motivo: '||p_motivo||', p_tipo_gasto: '||p_tipo_gasto;
  
 --Verifica que todos los campos obligatorios tienen valor
	if (p_cod_pedido is NULL or p_cod_presupuesto is NULL or p_fecha_aprobacion is NULL or p_tipo_gasto is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

  -- Valida que existe el pedido
  BEGIN
    select 	count(cod_pedido)
    into	num_ped
    from	INV_PEDIDO
    where cod_pedido = p_cod_pedido and esta_autorizado_ped=1; --el pedido debe estar aprobado por el resp. de dept.
  END;
  if (num_ped = 0) then error_desc:='ERROR: El nmero de pedido indicado no existe o no est aprobado'; raise e_error; end if;
  -- Valida que existe el presupuesto
   BEGIN
    select 	count(cod_presup)
    into	num_presup
    from	INV_PRESUPUESTO
    where cod_presup = p_cod_presupuesto; 
  END;
  if (num_presup = 0) then error_desc:='ERROR: El presupuesto indicado no existe'; raise e_error; end if;
  
    -- Valida que existe el empleado responsable de aprobar la compra
   BEGIN
    select 	count(cod_emp)
    into	num_emp
    from	INV_EMPLEADO
    where cod_emp = p_emp_responsable; 
  END;
  if (num_emp = 0) then error_desc:='ERROR: El responsable de aprobar la compra no existe'; raise e_error; end if;
  
  -- Se considerar que en la misma fecha que se aprueba la compra se realiza la peticin del pedido al proveedor

  update INV_PEDIDO set fec_ok_financiero_ped = p_fecha_aprobacion, fec_solicitud_ped=p_fecha_aprobacion,tipo_gasto_ped=p_tipo_gasto
  where cod_pedido = p_cod_pedido;
  
  if (p_motivo is not null) then
    update INV_PEDIDO set motivo_denega_ped = p_motivo where cod_pedido=p_cod_pedido;
  end if;
  
  -- Actualizamos el importe del pedido con el importe correspondiente del presupuesto ganador
  select importe_presup INTO importe_ganador from INV_PRESUPUESTO where cod_presup = p_cod_presupuesto;
  UPDATE INV_PEDIDO SET importe_ped = importe_ganador WHERE cod_pedido = p_cod_pedido;
  -- Actualizamos el pedido con la informacin del proveedor que lo servir y el responsable que aprob
  select cod_provee_presup INTO v_cod_proveedor from INV_PRESUPUESTO where cod_presup = p_cod_presupuesto;
  UPDATE INV_PEDIDO SET provee_ped = v_cod_proveedor, resp_financiero_ped = p_emp_responsable WHERE cod_pedido = p_cod_pedido;
  

  RSP := 'OK';
  -- Actualizamos el log con la accin realizada
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Aprueba compra]INV_PEDIDOS_PQ.INV_GEST_PED_APRUEBA_COMPRA_PR', parametrosIN, RSP,user);
  
  COMMIT; -- Confirmamos las operaciones
  
  -- Actualizamos estadsticas Q3

  if (p_tipo_gasto = 'I') then ESTADISTICAS_DWH_PQ.EST_Actualiza_importe_inv(TO_CHAR(p_fecha_aprobacion,'YYYY'),importe_ganador); end if;
  
  -- Tratamiento de excepciones
EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Aprueba compra]INV_PEDIDOS_PQ.INV_GEST_PED_APRUEBA_COMPRA_PR', parametrosIN, error_desc,user);
      COMMIT;
END;
						
-- Pedido recibido
PROCEDURE INV_GEST_PED_RECIBIDO_PR (
						p_cod_pedido IN INV_PEDIDO.cod_pedido%TYPE,
						p_fecha_recepcion IN DATE,
            RSP OUT varchar2) IS
  num_ped number;
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  v_proveedor number; --proveedor del pedido
  v_fecha_solicitud date;
  v_fecha_recepcion date;
  v_dias number; -- dias que han pasado hasta que se ha recibido el pedido
BEGIN

-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_pedido: '||p_cod_pedido||', p_fecha_recepcion: '||p_fecha_recepcion;
  
 --Verifica que todos los campos obligatorios tienen valor
	if (p_cod_pedido is NULL or p_fecha_recepcion is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

  -- Valida que existe el pedido
  BEGIN
    select 	count(cod_pedido)
    into	num_ped
    from	INV_PEDIDO
    where cod_pedido = p_cod_pedido and esta_autorizado_ped = 1; --el pedido debe estar aprobado por el resp. de dept.
  END;
  if (num_ped = 0) then error_desc:='ERROR: El nmero de pedido indicado no existe o no est aprobado'; raise e_error; end if;
  
  update INV_PEDIDO set fec_recepcion_ped = p_fecha_recepcion
  where cod_pedido = p_cod_pedido;


  RSP := 'OK';
  -- Actualizamos el log con la accin realizada
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Pedido recibido]INV_PEDIDOS_PQ.INV_GEST_PED_RECIBIDO_PR', parametrosIN, RSP,user);
  
  COMMIT; -- Confirmamos las operaciones
  
  -- Actualizamos las estadsticas de pedidos Q4
  BEGIN
    SELECT provee_ped,fec_solicitud_ped, fec_recepcion_ped
    INTO v_proveedor,v_fecha_solicitud,v_fecha_recepcion 
    FROM INV_PEDIDO 
    WHERE cod_pedido = p_cod_pedido;
  END;
  v_dias := v_fecha_recepcion - v_fecha_solicitud;
  ESTADISTICAS_DWH_PQ.EST_Nuevo_pedido_servido(v_proveedor, v_dias);

  -- Tratamiento de excepciones
EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Pedido recibido]INV_PEDIDOS_PQ.INV_GEST_PED_RECIBIDO_PR', parametrosIN, error_desc,user);
      COMMIT;
END;
            
            
            
	/*Descripcin:			 Alta de nuevo presupuesto
	Requisitos:				 Rol responsable financiero
	Parmetro entrada:
							*p_Fecha_presup
							*P_Validez_presup
							*P_Cod_proveedor_presup
							*P_Cod_pedido_presup
							P_Fecha_entrega
	Parmetro de salida
							RSP
							En caso de insercin con xito retorna el cdigo del identificador del
							presupuesto
	PRE-CONDICIN:			Existe un proveedor y un pedido
	POST-CONDICIN:		 	Se da de alta un presupuesto
							Se registra accin en la tabla LOG
							Devuelve Si todo ha ido bien:
							RSP = 'OK: Alta presupuesto con el Id = cod_ presupuesto
							Mensajes de error:
							'ERROR: hay campos obligatorios sin rellenar'*/

PROCEDURE INV_GEST_PED_ALTA_PRESUP_PR(
						p_fecha_presup IN DATE,
						p_validez_presup IN DATE,
						importe_presup NUMBER,
						p_cod_proveedor_presup IN INV_PROVEEDOR.cod_prov%TYPE,
						p_cod_pedido_presup IN INV_PEDIDO.cod_pedido%TYPE,
						--p_fecha_entrega IN DATE,
						RSP OUT varchar2,
						COD_NUEVO_PRESUP OUT number) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_ped number;
  num_prov number;
BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_fecha_presup: '||p_fecha_presup||', p_validez_presup: '||p_validez_presup||', importe_presup: '||importe_presup||', p_cod_proveedor_presup: '||p_cod_proveedor_presup||', p_cod_pedido_presup: '||p_cod_pedido_presup;
  
  -- VALIDACIONES
   -- Valida que existe el pedido
  BEGIN
    select 	count(cod_pedido)
    into	num_ped
    from	INV_PEDIDO
    where cod_pedido = p_cod_pedido_presup and esta_autorizado_ped=1; --el pedido debe estar aprobado por el resp. de dept.
  END;
  if (num_ped = 0) then error_desc:='ERROR: El nmero de pedido indicado no existe o no est aprobado'; raise e_error; end if;
  
  -- Valida que existe el proveedor
   BEGIN
    select 	count(cod_prov)
    into	num_prov
    from	INV_PROVEEDOR
    where cod_prov = p_cod_proveedor_presup; 
  END;
  if (num_prov = 0) then error_desc:='ERROR: El proveedor indicado no existe'; raise e_error; end if;
  
  INSERT INTO INV_PRESUPUESTO (cod_presup,fec_presup,importe_presup,validez_presup,cod_provee_presup,cod_pedido_presup)
  VALUES(presupuesto_seq.nextval,p_fecha_presup,importe_presup,p_validez_presup,p_cod_proveedor_presup,p_cod_pedido_presup);
  RSP := 'OK';
  COD_NUEVO_PRESUP := presupuesto_seq.currval;
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'[Alta Presupuesto]INV_PEDIDOS_PQ.INV_GEST_PED_ALTA_PRESUP_PR', parametrosIN, RSP,user);
  COMMIT;
  
EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
  if (error_desc is NULL) then error_desc := SQLCODE; end if;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'[Alta Presupuesto]INV_PEDIDOS_PQ.INV_GEST_PED_ALTA_PRESUP_PR', parametrosIN, error_desc,user);
  COMMIT; 
END INV_GEST_PED_ALTA_PRESUP_PR;

END INV_PEDIDOS_PQ; 
/