
-------------------------------------------------------------
---------- PACKAGE SPECIFICATION DISPOSITIVOS----------------
-------------------------------------------------------------


CREATE OR REPLACE PACKAGE INV_DISPOSITIVOS_PQ AS
  
	/*
	Descripcin:	 Alta de dispositivo en el momento de la recepcin 
	Requisitos: 	Rol administrador o Informtico o personal habilitado para recepcin 
	Parmetro entrada: 
					p_num_serie 
					p_fecha_validacion_inv 
					p_es_departamental 
					*p_fecha compra 
					p_comentario 
					p_cod_inventario 
					*p_cod_producto 
					*p_cod_empleado 
					*p_cod_sede -->>>AADIDO
	Parmetro de salida:  RSP 
					En caso de insercin con xito retorna el cdigo del identificador del dispositivo 
	PRE-CONDICIN: 	No existe el dispositivo. Existe el producto 
	POST-CONDICIN: Se da de alta el dispositivo 
					Se registra accin en la tabla LOG 
					Devuelve Si todo ha ido bien: RSP = 'OK: Alta dispositivo con el Id = cod_dispositivo' 
					Mensajes de error: 
									'ERROR: hay campos obligatorios sin rellenar' 
									'ERROR: El producto no existe en el sistema' 
									'ERROR: El dispositivo ya existe en el sistema' 
	*/
PROCEDURE INV_GEST_DISP_ALTA_PR (p_num_serie IN INV_DISPOSITIVO.num_serie_disp%TYPE,
								p_es_departamental IN INV_DISPOSITIVO.es_departamental_disp%TYPE,
								p_fecha_compra IN DATE,
								p_precio_compra_disp IN number,
								p_comentario IN INV_DISPOSITIVO.comentario_disp%type,
								p_cod_inventario IN INV_DISPOSITIVO.cod_inventario%TYPE,
								p_cod_producto IN INV_DISPOSITIVO.cod_producto%TYPE,
								p_cod_empleado IN INV_DISPOSITIVO.cod_empleado%TYPE,
								p_cod_sede IN INV_DISPOSITIVO.cod_sede%TYPE,
								RSP OUT VARCHAR2,
								COD_NUEVO_DISPOSITIVO OUT INV_DISPOSITIVO.cod_disp%TYPE);

	/*Descripcin: 			Baja de un dispositivo 
	Requisitos: 			Rol administrador o gestor funcional 
							El departamento debe estar dado de alta previamente 
	Parmetros entrada: 	*p_id_dispositivo: Identificador del dispositivo 
	Parmetro de salida: 	RSP
	PRE-CONDICIN :			Existe el departamento 
	POST-CONDICIN :		Se da de baja el departamento, aunque no se elimina el registro. 
							Simplemente queda en estado de baja 
							Se registra en el LOG 
							Devuelve Si todo ha ido bien: RSP = 'OK: Baja dispositivo realizada correctamente' 
							Mensajes de error: 
								'ERROR: hay campos obligatorios sin rellenar' 
								'ERROR: El dispositivo no existe en el sistema' 
	*/
PROCEDURE INV_GEST_DISP_BAJA_PR (p_id_dispositivo IN INV_DISPOSITIVO.cod_disp%TYPE,
                p_fecha_baja IN DATE,
                p_motivo_baja IN INV_MOVIMIENTO.desc_mov%TYPE,
				RSP OUT VARCHAR2);

	/*
	Descripcin: 			Mantenimiento de un dispositivo 
	Requisitos:				Rol administrador o gestor funcional 
							El dispositivo debe estar dado de alta previamente 
	Parmetro entrada:		p_cod_dispositivo 
							p_num_serie 
							p_fecha_validacion_inv 
							p_es_departamental 
							p_fecha_compra 
							p_comentario 
							p_cod_inventario 
							p_cod_producto 
							p_cod_empleado 
	Parmetro de salida 	RSP 
	PRE-CONDICIN 			Existe el dispositivo 
	POST-CONDICIN 			Se modifica el dispositivo. 
							Devuelve Si todo ha ido bien: RSP = 'OK: Actualizada la informacin del dispositivo' 
							Mensajes de error: 'ERROR: El dispositivo no existe en el sistema' 
	*/
PROCEDURE INV_GEST_DISP_MANT_PR (p_cod_dispositivo IN INV_DISPOSITIVO.cod_disp%TYPE,
								p_num_serie IN INV_DISPOSITIVO.num_serie_disp%TYPE,
								p_fecha_validacion_inv IN INV_DISPOSITIVO.fec_validacion_inv%TYPE,
								p_es_departamental IN INV_DISPOSITIVO.es_departamental_disp%TYPE,
								p_fecha_compra IN DATE,
								p_comentario IN INV_DISPOSITIVO.comentario_disp%TYPE,
								p_cod_inventario IN INV_DISPOSITIVO.cod_inventario%TYPE,
								p_cod_producto IN INV_DISPOSITIVO.cod_producto%TYPE,
								p_cod_empleado IN INV_DISPOSITIVO.cod_empleado%TYPE,
								p_cod_sede IN INV_DISPOSITIVO.cod_sede%TYPE, --AADIDO!!!!!
								RSP OUT VARCHAR2);

	/*
	Descripcin:			 Alta de nuevo movimiento 
	Requisitos:				 Rol administrador o Informtico  
	Parmetro entrada:		 *p_fecha_mov 
							 p_usu_origen_mov 
							 p_usu_destino_mov 
							 *p_cod_tipo_mov 
							 p_comentario 
	Parmetro de salida:	 RSP 
							 En caso de insercin con xito retorna el cdigo del identificador del nuevo movimiento 
	PRE-CONDICIN:			 Existe el dispositivo 
	POST-CONDICIN 			 Se da de alta el movimiento Se registra accin en la tabla LOG 
							 Devuelve Si todo ha ido bien: RSP = 'OK: Alta movimiento con el Id = cod_ movimiento ' 
							 Mensajes de error: 
								'ERROR: hay campos obligatorios sin rellenar' 
								'ERROR: El dispositivo no existe en el sistema' 
								'ERROR: El dispositivo ya existe en el sistema'  
	*/
PROCEDURE INV_GEST_DISP_ALTA_MOV_PR ( p_fecha_mov IN INV_MOVIMIENTO.fecha_mov%TYPE,
							p_cod_disp IN INV_MOVIMIENTO.cod_disp%TYPE,
							p_usu_origen_mov IN INV_MOVIMIENTO.usu_origen_mov%TYPE,
							p_usu_dest_mov IN INV_MOVIMIENTO.usu_dest_mov%TYPE,
							p_cod_tipo_mov IN INV_MOVIMIENTO.cod_tipo_mov%TYPE,
							p_comentario IN INV_MOVIMIENTO.desc_mov%TYPE,
							RSP OUT varchar2,
							COD_NUEVO_MOVIMIENTO OUT number); 
							
/*
	Descripcin: 			Baja de un movimiento 
	Requisitos:				Rol administrador o gestor funcional 
							El movimiento debe estar dado de alta previamente 
	Parmetro entrada:		p_cod_movimiento: cdigo identificador del movimiento 
	Parmetro de salida 	RSP 
	PRE-CONDICIN 			Existe el movimiento 
	POST-CONDICIN 			Se elimina el movimiento. 
							Devuelve: Si todo ha ido bien: RSP = 'OK: Eliminada la informacin del movimiento' 
							Mensajes de error: 'ERROR: El movimiento no existe en el sistema' 
	*/							
PROCEDURE INV_GEST_DISP_BAJA_MOV_PR (p_cod_movimiento IN INV_MOVIMIENTO.cod_mov%TYPE,
							RSP out VARCHAR2); 
/*modificacin movimiento*/	
PROCEDURE INV_GEST_DISP_MANT_MOV_PR ( p_cod_mov IN INV_MOVIMIENTO.cod_mov%TYPE,
							p_fecha_mov IN INV_MOVIMIENTO.fecha_mov%TYPE,
							p_usu_origen_mov IN INV_MOVIMIENTO.usu_origen_mov%TYPE,
							p_usu_dest_mov IN INV_MOVIMIENTO.usu_dest_mov%TYPE,
							p_cod_tipo_mov IN INV_MOVIMIENTO.cod_tipo_mov%TYPE,
							p_comentario IN INV_MOVIMIENTO.desc_mov%TYPE,
							RSP OUT varchar2);
							
							


END INV_DISPOSITIVOS_PQ; 
/