
-------------------------------------------------------------
--------------- PACKAGE BODY	 DISPOSITIVOS----------------
-------------------------------------------------------------


CREATE OR REPLACE PACKAGE BODY INV_DISPOSITIVOS_PQ AS
  
	/*
	Descripcin:	 Alta de dispositivo en el momento de la recepcin 
	Requisitos: 	Rol administrador o Informtico o personal habilitado para recepcin 
	Parmetro entrada: 
					p_num_serie 
					p_fecha_validacion_inv 
					p_es_departamental 
					*p_fecha compra 
					p_comentario 
					p_cod_inventario 
					*p_cod_producto 
					*p_cod_empleado 
					*p_cod_sede -->>>AADIDO
	Parmetro de salida:  RSP 
					En caso de insercin con xito retorna el cdigo del identificador del dispositivo 
	PRE-CONDICIN: 	No existe el dispositivo. Existe el producto 
	POST-CONDICIN: Se da de alta el dispositivo 
					Se registra accin en la tabla LOG 
					Devuelve Si todo ha ido bien: RSP = 'OK: Alta dispositivo con el Id = cod_dispositivo' 
					Mensajes de error: 
									'ERROR: hay campos obligatorios sin rellenar' 
									'ERROR: El producto no existe en el sistema' 
									'ERROR: El dispositivo ya existe en el sistema' 
	*/
PROCEDURE INV_GEST_DISP_ALTA_PR (p_num_serie IN INV_DISPOSITIVO.num_serie_disp%TYPE,
								p_es_departamental IN INV_DISPOSITIVO.es_departamental_disp%TYPE,
								p_fecha_compra IN DATE,
								p_precio_compra_disp IN number,
								p_comentario IN INV_DISPOSITIVO.comentario_disp%type,
								p_cod_inventario IN INV_DISPOSITIVO.cod_inventario%TYPE,
								p_cod_producto IN INV_DISPOSITIVO.cod_producto%TYPE,
								p_cod_empleado IN INV_DISPOSITIVO.cod_empleado%TYPE,
								p_cod_sede IN INV_DISPOSITIVO.cod_sede%TYPE,
								RSP OUT VARCHAR2,
								COD_NUEVO_DISPOSITIVO OUT INV_DISPOSITIVO.cod_disp%TYPE)
								IS
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  num_prod number;
  num_emp number;
  num_sede number;
  --V_COD_NUEVO_DISPOSITIVO number;
  V_COD_ALTA_MOV number;
  V_RSP varchar2(200);
BEGIN
-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_num_serie: '||p_num_serie||', p_es_departamental: '||p_es_departamental||', p_fecha_compra: '||p_fecha_compra||', precio_compra_disp: '|| p_precio_compra_disp||', p_comentario: '||p_comentario||', p_cod_inventario: '||p_cod_inventario||', p_cod_producto: '||p_cod_producto||', p_cod_empleado: '||p_cod_empleado||', p_cod_sede: '||p_cod_sede;
 --Verifica que todos los campos obligatorios tienen valor
	if (p_cod_producto is NULL or p_cod_sede is NULL or p_precio_compra_disp is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;

-- Verifica que existe el producto
  BEGIN
    select 	count(cod_prod)
    into	num_prod
    from	INV_PRODUCTO
    where cod_prod = p_cod_producto; 
  END;
  if (num_prod = 0) then error_desc:='ERROR: El producto indicado no existe'; raise e_error; end if;
  
  -- Verifica que no existe otro dispositivo con ese num de serie
  BEGIN
    select 	count(cod_disp)
    into	num_prod
    from	INV_DISPOSITIVO
    where num_serie_disp = p_num_serie; 
  END;
  if (num_prod = 1) then error_desc:='ERROR: No puede haber num. de serie duplicado!!!'; raise e_error; end if;
  
  -- Verifica que existe el empleado
  if p_cod_empleado is not null then
  BEGIN
    select 	count(cod_emp)
    into	num_emp
    from	INV_EMPLEADO
    where cod_emp = p_cod_empleado; 
  END;
  if (num_emp = 0) then error_desc:='ERROR: El producto indicado no existe'; raise e_error; end if;
  end if;
  
    -- Verifica que existe la sede
  BEGIN
    select 	count(cod_sede)
    into	num_sede
    from	INV_SEDE
    where cod_sede = p_cod_sede; 
  END;
  if (num_sede = 0) then error_desc:='ERROR: La sede indicada no existe'; raise e_error; end if;
  
    -- Aadimos nuevo dispositivo
    -- La primera vez que compramos el dispositivo le asignamos la misma fecha de inventario fsico = fecha compra
  INSERT INTO INV_DISPOSITIVO (cod_disp, num_serie_disp, fec_validacion_inv, es_departamental_disp,fec_compra_disp,precio_compra_disp,
  comentario_disp,cod_inventario,cod_producto,cod_empleado,cod_sede)
  VALUES (dispositivo_seq.nextval,p_num_serie,p_fecha_compra,p_es_departamental,p_fecha_compra,p_precio_compra_disp,
  p_comentario,p_cod_inventario,p_cod_producto,NULL,p_cod_sede); -- Inicialmente el dispositivo se registra, pero no pertenece a ningn empleado
  RSP := 'OK';
    -- Actualizamos el log con la accin realizada
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta dispositivo]INV_DISPOSITIVOS_PQ.INV_GEST_DISP_ALTA_PR', parametrosIN, RSP,user);
 
  COD_NUEVO_DISPOSITIVO := dispositivo_seq.currval;
  COMMIT;
  
  -- Al dar de alta el dispositivo adems se crea el primer movimiento: ALTA DISP.
  -- Se registra la informacin de quin recepcion el pedido (p_cod_empleado)
  
  INV_GEST_DISP_ALTA_MOV_PR(p_fecha_compra,COD_NUEVO_DISPOSITIVO,p_cod_empleado,NULL,'1','Alta producto introducida',V_RSP,V_COD_ALTA_MOV);

    -- ACTUALIZAMOS INFORMACIN ESTADSTICA Q1
  ESTADISTICAS_DWH_PQ.EST_Nuevo_dispositivo (EXTRACT(YEAR FROM p_fecha_compra)); 
  -- Actualizamos estadstica Q10
  ESTADISTICAS_DWH_PQ.EST_disp_almacenado(COD_NUEVO_DISPOSITIVO);
  
  -- Tratamiento de excepciones

EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta dispositivo]INV_DISPOSITIVOS_PQ.INV_GEST_DISP_ALTA_PR', parametrosIN, error_desc,user);
      COMMIT;
END INV_GEST_DISP_ALTA_PR;

	/*Descripcin: 			Baja de un dispositivo 
	Requisitos: 			Rol administrador o gestor funcional 
							El departamento debe estar dado de alta previamente 
	Parmetros entrada: 	*p_id_dispositivo: Identificador del dispositivo 
	Parmetro de salida: 	RSP
	PRE-CONDICIN :			Existe el departamento 
	POST-CONDICIN :		Se da de baja el departamento, aunque no se elimina el registro. 
							Simplemente queda en estado de baja 
							Se registra en el LOG 
							Devuelve Si todo ha ido bien: RSP = 'OK: Baja dispositivo realizada correctamente' 
							Mensajes de error: 
								'ERROR: hay campos obligatorios sin rellenar' 
								'ERROR: El dispositivo no existe en el sistema' 
	*/
PROCEDURE INV_GEST_DISP_BAJA_PR (p_id_dispositivo IN INV_DISPOSITIVO.cod_disp%TYPE,
                p_fecha_baja IN DATE,
                p_motivo_baja IN INV_MOVIMIENTO.desc_mov%TYPE,
								RSP OUT VARCHAR2) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  v_BAJA CONSTANT number := 2; 
BEGIN

  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_id_dispositivo: '||p_id_dispositivo;
  -- Lo dispositivos no se eliminan. Se anotan como un movimiento de tipo baja dispositivo
  
  BEGIN
    insert into INV_MOVIMIENTO (cod_mov,cod_disp,fecha_mov,usu_origen_mov,usu_dest_mov,cod_tipo_mov,desc_mov) 
    values (movimiento_seq.nextval,p_id_dispositivo,p_fecha_baja,NULL,NULL,v_BAJA, p_motivo_baja); --EN CASO DE BAJA NO ESPECIFICAMOS USUARIOS ORIGEN NI DESTINO
  END;
	RSP := 'OK';
  
	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja del dispositivo]INV_DISPOSITIVOS_PQ.INV_GEST_DISP_BAJA_PR', parametrosIN, RSP,user);
	
	COMMIT;
	
EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
    if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja del dispositivo]INV_DISPOSITIVOS_PQ.INV_GEST_DISP_BAJA_PR', parametrosIN, error_desc,user);
  COMMIT;  
  NULL;
END INV_GEST_DISP_BAJA_PR;

	/*
	Descripcin: 			Mantenimiento de un dispositivo 
	Requisitos:				Rol administrador o gestor funcional 
							El dispositivo debe estar dado de alta previamente 
	Parmetro entrada:		p_cod_dispositivo 
							p_num_serie 
							p_fecha_validacion_inv 
							p_es_departamental 
							p_fecha_compra 
							p_comentario 
							p_cod_inventario 
							p_cod_producto 
							p_cod_empleado 
	Parmetro de salida 	RSP 
	PRE-CONDICIN 			Existe el dispositivo 
	POST-CONDICIN 			Se modifica el dispositivo. 
							Devuelve Si todo ha ido bien: RSP = 'OK: Actualizada la informacin del dispositivo' 
							Mensajes de error: 'ERROR: El dispositivo no existe en el sistema' 
	*/
PROCEDURE INV_GEST_DISP_MANT_PR (p_cod_dispositivo IN INV_DISPOSITIVO.cod_disp%TYPE,
								p_num_serie IN INV_DISPOSITIVO.num_serie_disp%TYPE,
								p_fecha_validacion_inv IN INV_DISPOSITIVO.fec_validacion_inv%TYPE,
								p_es_departamental IN INV_DISPOSITIVO.es_departamental_disp%TYPE,
								p_fecha_compra IN DATE,
								p_comentario IN INV_DISPOSITIVO.comentario_disp%TYPE,
								p_cod_inventario IN INV_DISPOSITIVO.cod_inventario%TYPE,
								p_cod_producto IN INV_DISPOSITIVO.cod_producto%TYPE,
								p_cod_empleado IN INV_DISPOSITIVO.cod_empleado%TYPE,
								p_cod_sede IN INV_DISPOSITIVO.cod_sede%TYPE, --AADIR EN LA ESPECIFICACION
								RSP OUT VARCHAR2) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_disp number;

BEGIN
	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_dispositivo: '||p_cod_dispositivo||', p_num_serie: '||p_num_serie||', p_fecha_validacion_inv: '||p_fecha_validacion_inv||', p_es_departamental: '||p_es_departamental||', p_fecha_compra: '||p_fecha_compra||', p_comentario: '||p_comentario||', p_cod_inventario: '||p_cod_inventario||', p_cod_producto: '||p_cod_producto||', p_cod_empleado: '||p_cod_empleado||', p_cod_sede: '||p_cod_sede;
	
	 --Verifica que todos los campos obligatorios tienen valor
	if (p_cod_dispositivo is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;
	
	-- Verifica que existe el dispositivo
	BEGIN
		SELECT 	count(cod_disp)
		INTO	num_disp
		FROM	INV_DISPOSITIVO
		WHERE	cod_disp = p_cod_dispositivo; 
	END;
	if (num_disp = 0) then error_desc:='ERROR: El dispositivo indicado no existe'; raise e_error; end if;
	
	UPDATE inv_dispositivo
	SET	num_serie_disp = NVL(p_num_serie,num_serie_disp),
		fec_validacion_inv = NVL(p_fecha_validacion_inv,fec_validacion_inv),
		es_departamental_disp = NVL(p_es_departamental,es_departamental_disp),
		fec_compra_disp = NVL(p_fecha_compra,fec_compra_disp),
		comentario_disp	= NVL(p_comentario,comentario_disp),
		cod_inventario	= NVL(p_cod_inventario,cod_inventario),
		cod_producto = NVL(p_cod_producto,cod_producto),
		cod_empleado = NVL(p_cod_empleado,cod_empleado),
		cod_sede = NVL(p_cod_sede,cod_sede)
	WHERE cod_disp = p_cod_dispositivo;
    RSP := 'OK';
	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Mod del dispositivo]INV_DISPOSITIVOS_PQ.INV_GEST_DISP_MANT_PR', parametrosIN, RSP,user);	
	
	COMMIT;
		
		
EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Mod del dispositivo]INV_DISPOSITIVOS_PQ.INV_GEST_DISP_MANT_PR', parametrosIN, error_desc,user);
	  COMMIT;  

	  
END INV_GEST_DISP_MANT_PR;

	/*
	Descripcin:			 Alta de nuevo movimiento 
	Requisitos:				 Rol administrador o Informtico  
	Parmetro entrada:		 *p_fecha_mov 
							 p_usu_origen_mov 
							 p_usu_destino_mov 
							 *p_cod_tipo_mov 
							 p_comentario 
	Parmetro de salida:	 RSP 
							 En caso de insercin con xito retorna el cdigo del identificador del nuevo movimiento 
	PRE-CONDICIN:			 Existe el dispositivo 
	POST-CONDICIN 			 Se da de alta el movimiento Se registra accin en la tabla LOG 
							 Devuelve Si todo ha ido bien: RSP = 'OK: Alta movimiento con el Id = cod_ movimiento ' 
							 Mensajes de error: 
								'ERROR: hay campos obligatorios sin rellenar' 
								'ERROR: El dispositivo no existe en el sistema' 
								'ERROR: El dispositivo ya existe en el sistema'  
	*/
PROCEDURE INV_GEST_DISP_ALTA_MOV_PR ( p_fecha_mov IN INV_MOVIMIENTO.fecha_mov%TYPE,
              p_cod_disp IN INV_MOVIMIENTO.cod_disp%TYPE,
							p_usu_origen_mov IN INV_MOVIMIENTO.usu_origen_mov%TYPE,
							p_usu_dest_mov IN INV_MOVIMIENTO.usu_dest_mov%TYPE,
							p_cod_tipo_mov IN INV_MOVIMIENTO.cod_tipo_mov%TYPE,
							p_comentario IN INV_MOVIMIENTO.desc_mov%TYPE,
							RSP OUT varchar2,
							COD_NUEVO_MOVIMIENTO OUT number) IS
  parametrosIN varchar2(1000);
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  num_tipo_mov number;
  num_emp number;
  num_disp number;
  es_portatil number;
  v_usuario_origen number;
  v_usuario_destinatario number;
  v_rsp varchar2(200);
  v_depart_destino number;
  v_depart_origen number;
  v_valor_disp number;
BEGIN
	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_fecha_mov: '||p_fecha_mov||', p_cod_disp: '||p_cod_disp||', p_usu_origen_mov: '||p_usu_origen_mov||', p_usu_dest_mov: '||p_usu_dest_mov||', p_cod_tipo_mov: '||p_cod_tipo_mov||', p_comentario: '||p_comentario;
	--Verifica que todos los campos obligatorios tienen valor
	if (p_fecha_mov is NULL or p_cod_tipo_mov is NULL or p_cod_disp is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;
  
  
  -- Verifica que existe el tipo de movimiento
  BEGIN
    select 	count(cod_tipo_mov)
    into	num_tipo_mov
    from	INV_TIPO_MOVIMIENTO
    where cod_tipo_mov = p_cod_tipo_mov; 
  END;
  if (num_tipo_mov = 0) then error_desc:='ERROR: El tipo de movimiento no existe'; raise e_error; end if;


  -- Verifica que existe el empleado origen
  if (p_usu_origen_mov is not null) then
    BEGIN
      select 	count(cod_emp)
      into	num_emp
      from	INV_EMPLEADO
      where cod_emp = p_usu_origen_mov; 
    END;
    if (num_emp = 0) then error_desc:='ERROR: El empleado origen indicado no existe'; raise e_error; end if;
  end if;
  
    -- Verifica que existe el empleado destino
  if (p_usu_dest_mov is not null) then
    BEGIN
      select 	count(cod_emp)
      into	num_emp
      from	INV_EMPLEADO
      where cod_emp = p_usu_dest_mov; 
    END;
    if (num_emp = 0) then error_desc:='ERROR: El empleado destino indicado no existe'; raise e_error; end if;
  end if;
  
  -- Verifica que existe el dispositivo
  BEGIN
    select 	count(cod_disp)
    into	num_disp
    from	INV_DISPOSITIVO
    where cod_disp = p_cod_disp; 
  END;
  if (num_disp = 0) then error_desc:='ERROR: El dispositivo indicado no existe'; raise e_error; end if;
  
  
    -- si es de tipo asignacin o reasignacin el campo de usuarios destinatario es obligatorio

  if (p_cod_tipo_mov = '3' or p_cod_tipo_mov = '4') then
    if (p_usu_dest_mov = NULL) then
      error_desc := 'Para este tipo de movimiento se deben indicar los usuarios afectados';
      raise e_error;
    end if;
  end if;
  -- para una reasignacin o avera
  -- Adems, el usuario origen debe ser el actual propietario del dispositivo
  if (p_cod_tipo_mov = '4' or p_cod_tipo_mov = '5') then
    v_usuario_origen := AUXILIAR_PQ.empleado_prop_dispositivo(p_cod_disp);
  end if;
  
  
    -- Aadimos nuevo movimiento
  INSERT INTO INV_MOVIMIENTO (cod_mov,cod_disp,fecha_mov,usu_origen_mov,usu_dest_mov,cod_tipo_mov,desc_mov)
  VALUES (movimiento_seq.nextval,p_cod_disp,p_fecha_mov,v_usuario_origen,p_usu_dest_mov,p_cod_tipo_mov,p_comentario);
  RSP := 'OK'; 
  COD_NUEVO_MOVIMIENTO := dispositivo_seq.currval;
  
   -- Actualizamos el log con la accin realizada
  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
  VALUES(log_seq.nextval, SYSDATE,'([Alta movimiento: '||p_cod_tipo_mov||']INV_DISPOSITIVOS_PQ.INV_GEST_DISP_ALTA_MOV_PR', parametrosIN, RSP,user);
  
  COMMIT; 
  
  -----------------------------
  -- ACTUALIZAMOS EL PROPIETARIO DEL DISPOSITIVO
  -- segn el momiento realizado si es asigna, reasigna, baja, o retirada actualiza dispositivo.usu_asignado

  if (p_cod_tipo_mov = '3' or p_cod_tipo_mov = '4') then --asignacin o reasignacin
    INV_GEST_DISP_MANT_PR(p_cod_disp,NULL,NULL,NULL,NULL,NULL,NULL,NULL,p_usu_dest_mov,NULL,v_rsp);
  elsif (p_cod_tipo_mov = '2' or p_cod_tipo_mov = '6') then --baja o retirada
    v_usuario_destinatario := NULL;
    INV_GEST_DISP_MANT_PR(p_cod_disp,NULL,NULL,NULL,NULL,NULL,NULL,NULL,v_usuario_destinatario,NULL,v_rsp);
  end if;
  -- Si el movimiento es avera es necesaro que el "usuario origen" sea igual al propietario actual
  if (p_cod_tipo_mov = '5') then -- dispositivo averiado
    INV_GEST_DISP_MANT_PR(p_cod_disp,NULL,NULL,NULL,NULL,NULL,NULL,v_usuario_origen,NULL,NULL,v_rsp);
  end if;
  ---------------------------
 
  -- ACTUALIZAMOS ESTADISTICAS

  -- Actualizamos estadsticas Q13
  if (p_cod_tipo_mov = '5') then -- dispositivo averiado
    ESTADISTICAS_DWH_PQ.EST_Nueva_averia(EXTRACT (YEAR FROM p_fecha_mov));
    ESTADISTICAS_DWH_PQ.EST_Calcula_averias;
  end if;
  
  es_portatil := AUXILIAR_PQ.es_un_portatil(p_cod_disp);
  if (p_cod_tipo_mov = '5' and es_portatil=1) then --averia y porttil
    ESTADISTICAS_DWH_PQ.EST_Nueva_Averia_portatil(p_cod_disp); 
  end if;
  -- Actualizamos estadsticas Q2
  if (p_cod_tipo_mov = '1' and es_portatil=1) then --alta porttil
      ESTADISTICAS_DWH_PQ.EST_Nuevo_portatil;
  end if;
  if (p_cod_tipo_mov = '2' and es_portatil=1) then --baja porttil
      ESTADISTICAS_DWH_PQ.EST_Baja_portatil;
  end if;

  -- Actualizamos estadsticas Q5

  if (p_cod_tipo_mov = '3')  then --asignacin
    v_depart_destino := AUXILIAR_PQ.departamento_asignado(p_usu_dest_mov);
    ESTADISTICAS_DWH_PQ.EST_Nuevo_dispositivo_en_uso(v_depart_destino);
    v_valor_disp := AUXILIAR_PQ.valor_econ_dispositivo(p_cod_disp);
    ESTADISTICAS_DWH_PQ.EST_Suma_valor_inv_activo(v_depart_destino,v_valor_disp);
    ESTADISTICAS_DWH_PQ.EST_Nueva_asignacion_disp(p_usu_dest_mov);
    ESTADISTICAS_DWH_PQ.EST_disp_entregado(p_cod_disp);
    ESTADISTICAS_DWH_PQ.EST_suma_disp_diferente(v_depart_destino,p_cod_disp); 

  elsif (p_cod_tipo_mov = '4') then --reasignacin
    null;
    v_depart_origen := AUXILIAR_PQ.departamento_asignado(v_usuario_origen);
    ESTADISTICAS_DWH_PQ.EST_Nueva_reasignacion(v_depart_origen); --la contabilizamos en el origen
    ESTADISTICAS_DWH_PQ.EST_Retira_dispositivo_en_uso(v_depart_origen);
    v_depart_destino := AUXILIAR_PQ.departamento_asignado(p_usu_dest_mov);
    ESTADISTICAS_DWH_PQ.EST_Nuevo_dispositivo_en_uso(v_depart_destino);
    v_valor_disp := AUXILIAR_PQ.valor_econ_dispositivo(p_cod_disp);
    ESTADISTICAS_DWH_PQ.EST_Resta_valor_inv_activo(v_depart_origen,v_valor_disp);
    ESTADISTICAS_DWH_PQ.EST_Suma_valor_inv_activo(v_depart_destino,v_valor_disp);
    
    ESTADISTICAS_DWH_PQ.EST_Nueva_asignacion_disp(p_usu_dest_mov);
    ESTADISTICAS_DWH_PQ.EST_Retirada_dipositivo(v_usuario_origen);
    
    ESTADISTICAS_DWH_PQ.EST_suma_disp_diferente(v_depart_destino,p_cod_disp); 
    ESTADISTICAS_DWH_PQ.EST_resta_disp_diferente(v_depart_origen,p_cod_disp); 
    
  elsif (p_cod_tipo_mov = '2' or p_cod_tipo_mov = '6') then --baja o retirado
    v_depart_origen := AUXILIAR_PQ.departamento_asignado(p_usu_origen_mov);
    ESTADISTICAS_DWH_PQ.EST_Retira_dispositivo_en_uso(v_depart_origen);
    
    v_valor_disp := AUXILIAR_PQ.valor_econ_dispositivo(p_cod_disp);
    ESTADISTICAS_DWH_PQ.EST_Resta_valor_inv_activo(v_depart_origen,v_valor_disp);
    ESTADISTICAS_DWH_PQ.EST_resta_disp_diferente(v_depart_origen,p_cod_disp); 
  end if;
  
  -- Estadisticas Q10
  if (p_cod_tipo_mov='2') then --Baja
        ESTADISTICAS_DWH_PQ.EST_disp_entregado(p_cod_disp);
  elsif (p_cod_tipo_mov='6') then --retirado = almacenado
        ESTADISTICAS_DWH_PQ.EST_disp_almacenado(p_cod_disp);
  end if;
  
  COMMIT; -- Confirmamos las operaciones

  -- Tratamiento de excepciones

EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
      ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
      VALUES(log_seq.nextval, SYSDATE,'([Alta movimiento: '||p_cod_tipo_mov||']INV_DISPOSITIVOS_PQ.INV_GEST_DISP_ALTA_MOV_PR', parametrosIN, error_desc,user);
      COMMIT;
END INV_GEST_DISP_ALTA_MOV_PR;
							
/*
	Descripcin: 			Baja de un movimiento 
	Requisitos:				Rol administrador o gestor funcional 
							El movimiento debe estar dado de alta previamente 
	Parmetro entrada:		p_cod_movimiento: cdigo identificador del movimiento 
	Parmetro de salida 	RSP 
	PRE-CONDICIN 			Existe el movimiento 
	POST-CONDICIN 			Se elimina el movimiento. 
							Devuelve: Si todo ha ido bien: RSP = 'OK: Eliminada la informacin del movimiento' 
							Mensajes de error: 'ERROR: El movimiento no existe en el sistema' 
	*/							
PROCEDURE INV_GEST_DISP_BAJA_MOV_PR (p_cod_movimiento IN INV_MOVIMIENTO.cod_mov%TYPE,
							RSP out VARCHAR2) IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
BEGIN
  -- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_movimiento: '||p_cod_movimiento;
    BEGIN
		delete from INV_MOVIMIENTO where cod_mov = p_cod_movimiento;
	END;
	RSP := 'OK';
  
	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Baja movimiento]INV_DISPOSITIVOS_PQ.INV_GEST_DISP_BAJA_MOV_PR', parametrosIN, RSP,user);
	
	COMMIT;
	
EXCEPTION  
WHEN OTHERS THEN  -- handles all errors
  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
		INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
    VALUES(log_seq.nextval, SYSDATE,'([Baja movimiento]INV_DISPOSITIVOS_PQ.INV_GEST_DISP_BAJA_MOV_PR', parametrosIN, error_desc,user);
  COMMIT;
END INV_GEST_DISP_BAJA_MOV_PR;
							

/*
	Descripcin: 			Modificacin de un movimiento existente
	NOTA: No actualiza estadsticas
*/	
PROCEDURE INV_GEST_DISP_MANT_MOV_PR ( p_cod_mov IN INV_MOVIMIENTO.cod_mov%TYPE,
							p_fecha_mov IN INV_MOVIMIENTO.fecha_mov%TYPE,
							p_usu_origen_mov IN INV_MOVIMIENTO.usu_origen_mov%TYPE,
							p_usu_dest_mov IN INV_MOVIMIENTO.usu_dest_mov%TYPE,
							p_cod_tipo_mov IN INV_MOVIMIENTO.cod_tipo_mov%TYPE,
							p_comentario IN INV_MOVIMIENTO.desc_mov%TYPE,
							RSP OUT varchar2)  IS
  error_desc varchar2(1000); -- descripcion del error
  e_error exception; --error controlado
  parametrosIN varchar2(1000);
  num_mov number;

BEGIN
	-- Concatenamos los parametros de entrada para el LOG
	parametrosIN := 'p_cod_mov: '||p_cod_mov||', p_fecha_mov: '||p_fecha_mov||', p_usu_origen_mov: '||p_usu_origen_mov||', p_usu_dest_mov: '||p_usu_dest_mov||', p_cod_tipo_mov: '||p_cod_tipo_mov||', p_comentario: '||p_comentario;
	
	 --Verifica que todos los campos obligatorios tienen valor
	if (p_cod_mov is NULL) then 
	 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
	 raise e_error;
	end if;
	
	-- Verifica que existe el movimiento que se va a actualizar
	BEGIN
		SELECT 	count(cod_mov)
		INTO	num_mov
		FROM	INV_MOVIMIENTO
		WHERE	cod_mov = p_cod_mov; 
	END;
	if (num_mov = 0) then error_desc:='ERROR: El movimiento indicado no existe'; raise e_error; end if;
	
	-- Actualiza valores del movimiento indicado
	UPDATE inv_movimiento
	SET	fecha_mov = NVL(p_fecha_mov,fecha_mov),
		usu_origen_mov = NVL(p_usu_origen_mov,usu_origen_mov),
		usu_dest_mov = NVL(p_usu_dest_mov,usu_dest_mov),
		cod_tipo_mov = NVL(p_cod_tipo_mov,cod_tipo_mov),
		desc_mov = NVL(p_comentario,desc_mov)
	WHERE cod_mov = p_cod_mov;

	INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	VALUES(log_seq.nextval, SYSDATE,'([Modificacin del movimiento]INV_DISPOSITIVOS_PQ.INV_GEST_MANT_MOV_PR', parametrosIN, 'OK',user);	
	RSP:='OK';
	COMMIT;
		
		
EXCEPTION  
	WHEN OTHERS THEN  -- handles all errors
	  ROLLBACK;
      if (error_desc is NULL) then error_desc := SQLCODE ||', MSG:' ||SUBSTR(SQLERRM, 1, 100) ; end if;
			INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		VALUES(log_seq.nextval, SYSDATE,'([Modificacin del dispositivo]INV_DISPOSITIVOS_PQ.INV_GEST_MANT_MOV_PR', parametrosIN, error_desc,user);
	  COMMIT;  
	  NULL;
	  
END INV_GEST_DISP_MANT_MOV_PR;
							

END INV_DISPOSITIVOS_PQ; 
/