CREATE OR REPLACE PACKAGE BODY INVENTARIO_PQ AS

	/*
	Descripcin:		 Alta inventario
	Requisitos:			 Rol administrador o rol encargado de inventario
	Parmetro entrada:
						*P_Sede_inventario
						*P_Fecha_inventario
						*Empleado_responsable_inv
	Parmetro de salida
						Si todo ha ido bien:
						RSP = 'OK: Alta inventario con el Id = cod_inventario '
						Mensajes de error:
							'ERROR: hay campos obligatorios sin rellenar'
							'ERROR: El dispositivo no existe en el sistema'
							'ERROR: El dispositivo ya existe en el sistema'
	PRE-CONDICIN
	POST-CONDICIN 		RSP
						Registra accin en el log del sistema*/

PROCEDURE INV_GEST_INV_ALTA_PR (
                          p_sede_inventario IN INV_INVENTARIO.sede_inv%TYPE,
                          p_fecha_inv IN DATE,
                          p_empleado_responsable IN INV_INVENTARIO.emp_responsable_inv%TYPE,
                          RSP OUT VARCHAR2,
                          COD_NUEVO_INV OUT INV_INVENTARIO.cod_inv%TYPE) IS
	  parametrosIN varchar2(1000);
	  num_emp number;
	  num_sede number;
	  error_desc varchar2(1000); -- descripcion del error
	  e_error exception; --error controlado
BEGIN
		-- Concatenamos los parametros de entrada para el LOG
		parametrosIN := 'p_sede_inventario: '||p_sede_inventario||', p_fecha_inv: '||p_fecha_inv||', p_empleado_responsable;: '||p_empleado_responsable;

	 --Verifica que todos los campos obligatorios tienen valor
		if (p_sede_inventario is NULL or p_fecha_inv is NULL or p_empleado_responsable is NULL) then 
		 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
		 raise e_error;
		end if;

	  --Verifica que el empleado existe en la Base de Dades
	  BEGIN
      select 	count(cod_emp)
      into	num_emp
      from	inv_empleado
      where cod_emp = p_empleado_responsable; 
	  END;
	  if (num_emp = 0) then error_desc:='ERROR: El empleado indicado no existe'; raise e_error; end if;
	 
	  --Verifica que la sede existe en la Base de Dades
	  
	  BEGIN
      select count(cod_sede)
      into	num_sede
      from	inv_sede
      where cod_sede = p_sede_inventario; 
	  END;
	  if (num_sede = 0) then error_desc:='ERROR: La sede indicada no existe'; raise e_error; end if;
	  
	  -- Insertamos nuevo inventario
	  INSERT INTO INV_INVENTARIO (cod_inv,sede_inv,fecha_inv,emp_responsable_inv)
	  VALUES (inventario_seq.nextval,p_sede_inventario,p_fecha_inv,p_empleado_responsable);

	  RSP := 'OK';
	  COD_NUEVO_INV := inventario_seq.currval;
			
	  -- Actualizamos el log con la accin realizada
	  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	  VALUES(log_seq.nextval, SYSDATE,'([Alta inventario]INVENTARIO_PQ.INV_GEST_INV_ALTA_PR', parametrosIN, RSP,user);
	  
	  COMMIT; -- Confirmamos las operaciones
	  
	  -- Tratamiento de excepciones

	EXCEPTION  
		WHEN OTHERS THEN  -- handles all errors
		  ROLLBACK;
		  if (error_desc is NULL) then error_desc := SQLCODE; end if;
				INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		  VALUES(log_seq.nextval, SYSDATE,'([Alta inventario]INVENTARIO_PQ.INV_GEST_INV_ALTA_PR', parametrosIN, error_desc,user);
		  COMMIT;
		
  
	END INV_GEST_INV_ALTA_PR;

	  
	/*Descripcin:		 Alta dispositivo en inventario
	Requisitos:			 Rol administrador o rol encargado de inventario
	Parmetro entrada:
						*P_cod_inventario
						*p_cod_dispositivo
	Parmetro de salida
						RSP
	PRE-CONDICIN
	POST-CONDICIN 		Registra accin en el log del sistema
						Registra el dispositivo en un inventario
						Devuelve 
							Si todo ha ido bien:
								RSP = 'OK: Alta dispositivo en inventario '
							Mensajes de error:
							'ERROR: hay campos obligatorios sin rellenar'
							'ERROR: El dispositivo no existe en el sistema'
							'ERROR: El inventario no existe en el sistema'
	*/

	  -- INVENTARIA DISPOSITIVO
	PROCEDURE INV_GEST_INVENTARIA_DISP_PR(
                    p_cod_inventario IN INV_INVENTARIO.cod_inv%TYPE,
										p_cod_dispositivo IN INV_DISPOSITIVO.cod_disp%TYPE,
                    RSP OUT VARCHAR2) IS
	  parametrosIN varchar2(1000);
	  v_num_inv number;
	  v_num_disp number;
	  error_desc varchar2(1000); -- descripcion del error
	  e_error exception; --error controlado
	BEGIN
		-- Concatenamos los parametros de entrada para el LOG
		parametrosIN := 'P_cod_inventario: '||P_cod_inventario||', P_cod_dispositivo: '||P_cod_dispositivo;

	 --Verifica que todos los campos obligatorios tienen valor
		if (p_cod_inventario is NULL or P_cod_dispositivo is NULL) then 
		 error_desc:='ERROR: Parmetro obligatorio con valor nulo';
		 raise e_error;
		end if;

	  --Verifica que el inventario existe en la Base de Dades
	  BEGIN
      select 	count(cod_inv)
      into	v_num_inv
      from	inv_inventario
      where cod_inv = P_cod_inventario; 
	  END;
	  if (v_num_inv = 0) then error_desc:='ERROR: El inventario indicado no existe'; raise e_error; end if;
	 
	  --Verifica que el dispositivo existe en la Base de Dades
	  
	  BEGIN
      SELECT 	count(cod_disp)
      INTO	v_num_disp
      FROM	inv_dispositivo
      WHERE cod_disp = p_cod_dispositivo; 
	  END;
	  IF (v_num_disp = 0) THEN error_desc:='ERROR: La sede indicada no existe'; RAISE e_error; END IF;
	  
	  -- Actualizamos el dispositivo con la informacin de que ya est inventariado a la fecha actual
	  
	  UPDATE INV_DISPOSITIVO 
		SET COD_INVENTARIO = p_cod_inventario,
        FEC_VALIDACION_INV = SYSDATE
		WHERE cod_disp = p_cod_dispositivo;
	  --COD_NUEVO_INV := inventario_seq.currval;
	  RSP := 'OK';
			
	  -- Actualizamos el log con la accin realizada
	  INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
	  VALUES(log_seq.nextval, SYSDATE,'([Dispositivo inventariado]INVENTARIO_PQ.INV_GEST_INV_ALTA_DISPOSITIVO_PR', parametrosIN, RSP,user);
	  
	  COMMIT; -- Confirmamos las operaciones
	  
	  -- Tratamiento de excepciones
	EXCEPTION  
		WHEN OTHERS THEN  -- handles all errors
		  ROLLBACK;
		  if (error_desc is NULL) then error_desc := SQLCODE; end if;
				INSERT INTO INV_LOG (cod_log, fecha_log, funcion_log, param_in_log, param_out_log, usuario_log)
		  VALUES(log_seq.nextval, SYSDATE,'([Dispositivo inventariado]INVENTARIO_PQ.INV_GEST_INV_ALTA_DISPOSITIVO_PR', parametrosIN, error_desc,user);
		  COMMIT;
	END INV_GEST_INVENTARIA_DISP_PR;

END INVENTARIO_PQ;


